/**
 * EditCardGUI.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QAction;
import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QClipboard;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QColorDialog;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QFontDialog;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QInputDialog;
import com.trolltech.qt.gui.QKeyEvent;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QMainWindow;
import com.trolltech.qt.gui.QMenu;
import com.trolltech.qt.gui.QMenuBar;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QPainter;
import com.trolltech.qt.gui.QSizePolicy;
import com.trolltech.qt.gui.QToolBar;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;
import edu.jhu.cs.oose.elevel.MainWindow;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.buttons.DrawingButtonGroup;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.buttons.DrawingRadioButton;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.drawItems.DrawingType;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.dataModel.CardSection;
import edu.jhu.cs.oose.elevel.editing.EditCardWindow;
import edu.jhu.cs.oose.elevel.utilities.ClearingTextBox;
import edu.jhu.cs.oose.elevel.utilities.PreferenceCheckBox;
import edu.jhu.cs.oose.elevel.utilities.ViewState;

/**
 * GUI responsible for the view and controller for the underlying {@link EditCardWindow} model
 * 
 * @author Jesse Yates, H. Parker Shelton
 */
public class EditCardGUI extends QMainWindow
{
	/** The underlying {@link EditCardWindow} model */
	private EditCardWindow editWindow;

	/** The <code>Card</code> being edited */
	// private Card card;

	/** The card title */
	private ClearingTextBox title;

	/** The Question in displayable format */
	private DrawingGraphicsView question;
	private DrawingGraphicsScene questionModel;

	/** The answer in displayable format */
	private DrawingGraphicsView answer;
	private DrawingGraphicsScene answerModel;

	/** The hint in displayable format */
	private FocusingTextEdit hint;

	/** The hint in displayable format */
	private FocusingTextEdit addInfo;

	/** A {@link Map} representing the preferences of the <code>Card</code> */
	private Map<String, Boolean> pref;

	/** Physical widget holding the buttons */
	private QGroupBox drawingButtons;

	/** The toolbar used for file options */
	private QToolBar fileToolbar;

	/** Virtual construct grouping button actions together */
	private DrawingButtonGroup toolButtons;

	/** A QMenu representing the file menu */
	private QMenu fileMenu;

	/** A QMenu representing the edit menu */
	private QMenu editMenu;

	/** A QMenu representing the help menu */
	private QMenu helpMenu;

	/** A QAction that displays the about dialog */
	private QAction aboutAct;

	/** A QAction that saves a new card */
	private QAction saveNewAct;

	/** A QAction that saves the current <code>Card</code> */
	private QAction saveAct;

	/** A QAction that closes the current dialog */
	private QAction closeAct;

	/** Actions on the current editing area */
	private QAction setFontAct;
	private QAction setPenWidthAct;
	private QAction setPenColorAct;
	private QAction undoAct;
	private QAction redoAct;
	private QAction pasteAct;
	private QAction copyAct;
	private QAction cutAct;
	private QAction printAct;

	/** The main widget area */
	private QWidget editingArea;

	/** Signal that the window is closing */
	public final QSignalEmitter.Signal0 closing = new QSignalEmitter.Signal0();

	/** Clipboard for copy/pasting items into/out of scenes */
	private QClipboard clipboard;

	/**
	 * Constructor for the GUI
	 * 
	 * @param parent
	 *        a {@link MainWindow} representing the main window of the application
	 * @param edit
	 *        a {@link EditCardWindow} representing the underlying model for the GUI
	 */
	public EditCardGUI(MainWindow parent, EditCardWindow edit)
	{
		super(parent);

		this.setMenuBar(new QMenuBar());
		this.editWindow = edit;
		this.pref = new HashMap<String, Boolean>();
		this.pref.put("Display Hints", true);
		this.pref.put("Self-check Answer", true);

		// setup the window for display
		this.setupCard();
		this.setupToolbar();

		this.setupEditArea();

		setupActions();
		this.setupFileToolbar();
		setupMenu();

		this.clipboard = QApplication.clipboard();
	}

	/**
	 * Create the layout and generally format editing area
	 */
	private void setupEditArea()
	{
		editingArea = new QWidget();
		QVBoxLayout layout = new QVBoxLayout(editingArea);
		QGridLayout cardLayout = new QGridLayout();
		editingArea.setLayout(layout);

		cardLayout.setHorizontalSpacing(3);
		cardLayout.setVerticalSpacing(3);
		cardLayout.setMargin(5);
		// QVBoxLayout leftSide = new QVBoxLayout();
		// QVBoxLayout rightSide = new QVBoxLayout();
		// layout.addLayout(leftSide, 0, 0);
		// layout.addLayout(rightSide, 0, 1);

		QVBoxLayout settingsLayout = new QVBoxLayout();
		settingsLayout.addWidget(new QLabel("Settings"));
		for(Entry<String, Boolean> i : this.pref.entrySet())
		{
			PreferenceCheckBox box = new PreferenceCheckBox(i.getKey(), i.getKey());
			if(i.getValue().booleanValue())
				box.setCheckState(Qt.CheckState.Checked);
			box.stateChange.connect(this, "updateSetting(String, boolean)");
			settingsLayout.addWidget(box);
		}

		layout.addWidget(this.title, 1, Qt.AlignmentFlag.AlignCenter);
		layout.addLayout(cardLayout);

		cardLayout.setColumnMinimumWidth(0, CardSection.WIDTH);
		cardLayout.setColumnMinimumWidth(1, CardSection.WIDTH);
		cardLayout.setRowMinimumHeight(1, CardSection.HEIGHT);

		// cardLayout.setRowMinimumHeight()
		// layout.setRowMinimumHeight(0, CardSection.HEIGHT);
		// layout.setRowMinimumHeight(1, CardSection.HEIGHT);

		cardLayout.addWidget(new QLabel("Question"), 0, 0, Qt.AlignmentFlag.AlignCenter);
		cardLayout.addWidget(this.question, 1, 0);

		cardLayout.addWidget(new QLabel("Answer"), 0, 1, Qt.AlignmentFlag.AlignCenter);
		cardLayout.addWidget(this.answer, 1, 1);

		cardLayout.setRowMinimumHeight(2, 20);

		cardLayout.addWidget(new QLabel("Additional Information"), 3, 0, Qt.AlignmentFlag.AlignCenter);
		cardLayout.addWidget(this.addInfo, 4, 0);

		cardLayout.addWidget(new QLabel("Hint"), 3, 1, Qt.AlignmentFlag.AlignCenter);
		cardLayout.addWidget(this.hint, 4, 1);

		cardLayout.addLayout(settingsLayout, 5, 1, Qt.AlignmentFlag.AlignCenter);

		this.setCentralWidget(editingArea);
	}

	/**
	 * Initialize the actions in the menus
	 */
	private void setupActions()
	{
		this.saveAct = new QAction(tr("&Save"), this);
		this.saveAct.setShortcut(tr("Ctrl+S"));
		this.saveAct.setStatusTip(tr("Save the current card"));
		this.saveAct.triggered.connect(this, "saveCard()");

		this.saveNewAct = new QAction(tr("&Save + New Card"), this);
		this.saveNewAct.setStatusTip(tr("Save current card and start editing a new one."));
		this.saveNewAct.triggered.connect(this, "saveAndCreateNewCard()");

		this.closeAct = new QAction(tr("&Exit"), this);
		this.closeAct.setShortcut(tr("Ctrl+W"));
		this.closeAct.triggered.connect(this, "close()");

		this.aboutAct = new QAction(tr("&About"), this);
		this.aboutAct.setShortcut(tr("Ctrl+A"));
		this.aboutAct.triggered.connect(this, "about()");

		this.undoAct = new QAction(tr("&Undo"), this);
		this.undoAct.setShortcut(tr("Ctrl+Z"));
		this.undoAct.triggered.connect(this.answerModel, "undo()");
		this.undoAct.triggered.connect(this.questionModel, "undo()");
		// this.undoAct.triggered.connect(this.hint, "focusUndo()");
		// this.undoAct.triggered.connect(this.addInfo, "focusUndo()");

		this.redoAct = new QAction(tr("&Redo"), this);
		this.redoAct.setShortcut(tr("Ctrl+Y"));
		this.redoAct.triggered.connect(this.answerModel, "redo()");
		this.redoAct.triggered.connect(this.questionModel, "redo()");
		// this.redoAct.triggered.connect(this.hint, "focusRedo()");
		// this.redoAct.triggered.connect(this.addInfo, "focusRedo()");

		this.setPenColorAct = new QAction(tr("Pen Color"), this);
		this.setPenColorAct.triggered.connect(this, "setPenColor()");

		this.setPenWidthAct = new QAction(tr("Pen Width"), this);
		this.setPenWidthAct.triggered.connect(this, "setPenWidth()");

		this.setFontAct = new QAction(tr("Font"), this);
		this.setFontAct.triggered.connect(this, "setFont()");

		this.pasteAct = new QAction(tr("&Paste"), this);
		this.pasteAct.setShortcut(tr("Ctrl+V"));
		this.pasteAct.setStatusTip(tr("Paste data into editing area."));
		this.pasteAct.triggered.connect(this, "paste()");

		this.copyAct = new QAction(tr("&Copy"), this);
		this.copyAct.setShortcut(tr("Ctrl+C"));
		this.copyAct.setStatusTip(tr("Copy data from editing area."));
		this.copyAct.triggered.connect(this, "copy()");

		this.cutAct = new QAction(tr("&Cut"), this);
		this.cutAct.setShortcut(tr("Ctrl+X"));
		this.cutAct.setStatusTip(tr("Cut data from editing area."));
		this.cutAct.triggered.connect(this, "cut()");

		this.printAct = new QAction(tr("&Print"), this);
		this.printAct.setShortcut(tr("Ctrl+P"));
		this.printAct.setStatusTip(tr("Print the image from the editing area."));
		this.printAct.triggered.connect(this, "print()");

	}

	/**
	 * Get the information from the current {@link Card}
	 */
	private void setupCard()
	{
		CardSection questionSection = this.editWindow.getCard().getQuestion();
		CardSection answerSection = this.editWindow.getCard().getAnswer();
		CardSection hintSection = this.editWindow.getCard().getHint();
		CardSection addInfoSection = this.editWindow.getCard().getAdditionalInfo();
		String title = this.editWindow.getCard().getTitle();

		this.questionModel = questionSection.getScene();
		this.questionModel.setModified(false);
		this.question = new DrawingGraphicsView(this.questionModel);
		this.question.setRenderHints(QPainter.RenderHint.Antialiasing, QPainter.RenderHint.SmoothPixmapTransform);
		this.question.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff);
		this.question.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff);
		this.question.setBaseSize(CardSection.WIDTH, CardSection.HEIGHT);
		this.question.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed);
		this.question.setGeometry(0, 0, CardSection.WIDTH, CardSection.HEIGHT);

		this.answerModel = answerSection.getScene();
		this.answerModel.setModified(false);
		this.answer = new DrawingGraphicsView(this.answerModel);
		this.answer.setRenderHints(QPainter.RenderHint.Antialiasing, QPainter.RenderHint.SmoothPixmapTransform);
		this.answer.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff);
		this.answer.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff);
		this.answer.setBaseSize(CardSection.WIDTH, CardSection.HEIGHT);
		this.answer.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed);
		this.answer.setGeometry(0, 0, CardSection.WIDTH, CardSection.HEIGHT);

		if(hintSection.hasText())
		{
			this.hint = new FocusingTextEdit(hintSection.getText());
			this.hint.setMinimumHeight(20);
			this.hint.setTabChangesFocus(true);
		}
		else
			this.hint = new FocusingTextEdit("");

		if(addInfoSection.hasText())
		{
			this.addInfo = new FocusingTextEdit(addInfoSection.getText());
			this.addInfo.setMinimumHeight(20);
			this.addInfo.setTabChangesFocus(true);
		}
		else
			this.addInfo = new FocusingTextEdit("");

		if(!title.equals(""))
			this.title = new ClearingTextBox(title);
		else
			this.title = new ClearingTextBox("New Card");
		this.title.setAlignment(Qt.AlignmentFlag.AlignCenter);
		this.title.setMaximumHeight(30);

		this.pref = new HashMap<String, Boolean>();
		for(Entry<String, Boolean> j : this.editWindow.getCard().getOptions().entrySet())
			this.pref.put(new String(j.getKey()), new Boolean(j.getValue()));

		this.answerModel.gainedFocus.connect(this, "enableDrawingPreferences()");
		this.questionModel.gainedFocus.connect(this, "enableDrawingPreferences()");

		this.hint.gainedFocus.connect(this, "disableDrawingPreferences()");
		this.addInfo.gainedFocus.connect(this, "disableDrawingPreferences()");
	}

	/**
	 * Initialize the menus
	 */
	private void setupMenu()
	{
		this.fileMenu = menuBar().addMenu(tr("&File"));
		this.fileMenu.addAction(this.saveAct);
		// this.fileMenu.addAction(this.saveNewAct);
		this.fileMenu.addSeparator();
		this.fileMenu.addAction(this.printAct);
		this.fileMenu.addSeparator();
		this.fileMenu.addAction(this.closeAct);

		this.editMenu = menuBar().addMenu(tr("&Edit"));
		this.editMenu.addAction(pasteAct);
		this.editMenu.addAction(cutAct);
		this.editMenu.addAction(copyAct);
		this.editMenu.addSeparator();
		this.editMenu.addAction(undoAct);
		this.editMenu.addAction(redoAct);
		this.editMenu.addSeparator();
		this.editMenu.addAction(this.setFontAct);
		this.editMenu.addAction(this.setPenColorAct);
		this.editMenu.addAction(this.setPenWidthAct);

		menuBar().addSeparator();

		this.helpMenu = menuBar().addMenu(tr("&Help"));
		this.helpMenu.addAction(this.aboutAct);
	}

	private void setupFileToolbar()
	{
		fileToolbar = addToolBar(tr("File"));
		fileToolbar.addAction(saveAct);
	}

	/**
	 * Generate the toolbar and add to widget
	 */
	private void setupToolbar()
	{

		// System.out.println("Creating toolbar");
		this.toolButtons = new DrawingButtonGroup();
		DefaultToolBar toolbar = new DefaultToolBar();
		this.drawingButtons = new QGroupBox("Drawing style");
		DrawingRadioButton text = new DrawingRadioButton("Text", DrawingType.TEXT);
		text.setChecked(true);

		DrawingRadioButton line = new DrawingRadioButton("Line", DrawingType.LINE);
		DrawingRadioButton circle = new DrawingRadioButton("Circle", DrawingType.CIRCLE);
		DrawingRadioButton square = new DrawingRadioButton("Rectangle", DrawingType.RECTANGLE);
		DrawingRadioButton select = new DrawingRadioButton("Select", DrawingType.SELECT);
		QVBoxLayout layout = new QVBoxLayout();

		layout.addWidget(text);
		layout.addWidget(line);
		layout.addWidget(circle);
		layout.addWidget(square);
		layout.addWidget(select);

		this.toolButtons.addButton(text);
		this.toolButtons.addButton(line);
		this.toolButtons.addButton(circle);
		this.toolButtons.addButton(square);
		this.toolButtons.addButton(select);

		toolButtons.typeChange.connect(this.answer.scene(), "setType(DrawingItem)");
		toolButtons.typeChange.connect(this.question.scene(), "setType(DrawingItem)");

		// Set the default type to text
		toolButtons.typeChange.emit(DrawingType.TEXT.getItem());
		this.drawingButtons.setLayout(layout);
		toolbar.addWidget(this.drawingButtons);
		this.addToolBar(Qt.ToolBarArea.LeftToolBarArea, toolbar);
	}

	protected void keyPressEvent(QKeyEvent event)
	{
		if(event.key() == Qt.Key.Key_Escape.value())
			this.close();
	}

	/**
	 * Display information about editing a <code>Card</code>
	 */
	public void about()
	{
		QMessageBox.information(this, tr("Editing a Card"), tr("Different toolbars allow for different editing.<br>Simply put, input the question, answer, hint and additional information as you see fit, and then select the options for the card. Finally save the card."));
	}

	/**
	 * If the window attempts to close, determine if the card should be saved
	 * 
	 * @see com.trolltech.qt.gui.QWidget#closeEvent(com.trolltech.qt.gui.QCloseEvent)
	 */
	public void closeEvent(QCloseEvent event)
	{
		if(maybeSave())
		{
			event.accept();
			closing.emit();
		}
		else
			event.ignore();
	}

	/**
	 * Prompt the user to determine if the <code>Card</code> should be saved
	 * 
	 * @return <code>true</code> if the <code>Card</code> has been saved, <code>false</code> otherwise
	 */
	private boolean displaySavePrompt()
	{
		QMessageBox.StandardButton ret = QMessageBox.warning(this, tr("Edit Card"), "The card has been modified.\n" + "Save your changes?", new QMessageBox.StandardButtons(QMessageBox.StandardButton.Yes, QMessageBox.StandardButton.No, QMessageBox.StandardButton.Cancel));
		if(ret == QMessageBox.StandardButton.Yes)
		{
			saveCard();
			return true;
		}
		else if(ret == QMessageBox.StandardButton.Cancel)
		{
			return false;
		}

		// Revert card
		if(ViewState.INSTANCE.getCurrentCard().equals(new Card()))
			ViewState.INSTANCE.removeCurrentCard();
		else
			this.editWindow.revertCard();
		return true;
	}

	/**
	 * Returns the {@link EditCardWindow} model this GUI is associated with
	 * 
	 * @return the {@link EditCardWindow} model this GUI is associated with
	 */
	public EditCardWindow getEditWindow()
	{
		return this.editWindow;
	}

	/**
	 * Check to see the <code>card</code> should be saved
	 * 
	 * @return <code>true</code> if the <code>Card</code> has been saved, <code>false</code> otherwise
	 */
	private boolean maybeSave()
	{
		Card original = this.editWindow.getPreviousCard();
		Card current = this.editWindow.getCard();
		if(!current.equals(original))
		{
			if(current.equals(new Card()))
				return true;
		}
		// if(this.questionModel.equals(original.getQuestion().getScene()))
		// {
		// System.out.println("Question equals!");
		// if(this.answerModel.equals(original.getAnswer().getScene()))
		// {
		// System.out.println("Answer equals!");
		// if(this.hint.toPlainText().equals(original.getHint().getText()))
		// {
		// System.out.println("Hint equals!");
		// if(this.addInfo.toPlainText().equals(original.getAdditionalInfo().getText()))
		// {
		// System.out.println("Info equals!");
		// if(this.pref.equals(original.getOptions()))
		// {
		// System.out.println("OPtions equals!");
		// System.out.println(original.equals(new Card()));
		// if(!original.equals(new Card()))
		// {
		// System.out.println("Original not new!");
		// return true;
		// }
		// }
		// }
		// }
		// }
		// }
		return displaySavePrompt();
	}

	/**
	 * Save all the current values to the <code>Card</code>
	 */
	private void saveCard()
	{
		if(this.title.text().equals("New Card") || this.title.text().equals(""))
			this.editWindow.save(this.questionModel, this.answerModel, this.hint.toPlainText(), this.addInfo.toPlainText(), "New Card", this.pref);
		else
			this.editWindow.save(this.questionModel, this.answerModel, this.hint.toPlainText(), this.addInfo.toPlainText(), this.title.text(), this.pref);

		ViewState.INSTANCE.refreshCurrentCard();
	}

	@SuppressWarnings("unused")
	private void saveAndCreateNewCard()
	{
		saveCard();

		// TODO: is there a better way to re-initialize the edit card gui
		this.setMenuBar(new QMenuBar());
		this.pref = new HashMap<String, Boolean>();

		// setup the window for display
		this.setupCard();
		this.setupToolbar();

		this.setupEditArea();

		setupActions();
		this.setupFileToolbar();
		setupMenu();
	}

	/**
	 * Update the stored setting
	 * 
	 * @param k
	 *        The setting name
	 * @param v
	 *        The value to store
	 */
	public void updateSetting(String k, boolean v)
	{
		this.pref.put(k, new Boolean(v));
	}

	@SuppressWarnings("unused")
	private void setPenColor()
	{

		DrawingGraphicsScene model;
		if(this.answerModel.hasFocus() || this.questionModel.hasFocus())
		{
			System.out.println("Setting pen color!");
			if(this.answerModel.hasFocus())
			{
				model = this.answerModel;
				System.out.println("Model: answer");
			}
			else
			{
				model = this.questionModel;
				System.out.println("Model: question");
			}

			QColor newColor = QColorDialog.getColor(model.penColor());
			if(newColor.isValid())
				model.setPenColor(newColor);
		}
	}

	@SuppressWarnings("unused")
	private void setPenWidth()
	{
		System.out.println("Setting pen width!");
		DrawingGraphicsScene model = getFocusedScene();

		if(model != null)
		{
			Integer newWidth = QInputDialog.getInteger(this, tr("Edit Card"), tr("Select pen width:"), model.penWidth(), 1, 50, 1);
			if(newWidth != null)
				model.setPenWidth(newWidth.intValue());
		}
	}

	private DrawingGraphicsScene getFocusedScene()
	{
		if(this.answerModel.hasFocus())
		{
			System.out.println("Model: answer");
			return this.answerModel;
		}
		else if(this.questionModel.hasFocus())
		{
			System.out.println("Model: question");
			return this.questionModel;
		}
		return null;
	}

	@SuppressWarnings("unused")
	private void paste()
	{
		System.out.println("Pasting data into model!");
		DrawingGraphicsScene model = getFocusedScene();

		if(model != null)
		{
			model.paste(this.clipboard.mimeData());
		}
	}

	@SuppressWarnings("unused")
	private void cut()
	{
		System.out.println("Pasting data into model!");
		DrawingGraphicsScene model = getFocusedScene();

		if(model != null)
		{
			model.cut(this.clipboard);// (this.clipboard.mimeData());
		}
	}

	@SuppressWarnings("unused")
	private void copy()
	{
		System.out.println("Pasting data into model!");
		DrawingGraphicsScene model = getFocusedScene();

		if(model != null)
		{
			model.copy(this.clipboard);
		}
	}

	@SuppressWarnings("unused")
	private void print()
	{
		DrawingGraphicsScene model = getFocusedScene();

		if(model != null)
		{
			DrawingGraphicsView view = (DrawingGraphicsView) model.views().get(0);
			QDialog dialog = new QDialog();
			QVBoxLayout layout = new QVBoxLayout();
			QLabel label = new QLabel();
			label.setPixmap(view.getImage());
			layout.addWidget(label);
			dialog.setLayout(layout);
			dialog.exec();

		}
	}

	@SuppressWarnings("unused")
	private void setFont()
	{
		QWidget widget = this.focusWidget();
		System.out.println("Setting the font! from: " + widget.font());
		widget.setFont(QFontDialog.getFont(widget.font()).font);
		System.out.println("Setting font to: " + widget.font());
	}

	@SuppressWarnings("unused")
	private void disableDrawingPreferences()
	{
		// this.setFontAct.setEnabled(false);
		this.setPenColorAct.setEnabled(false);
		this.setPenWidthAct.setEnabled(false);
	}

	@SuppressWarnings("unused")
	private void enableDrawingPreferences()
	{
		// this.setFontAct.setEnabled(true);
		this.setPenColorAct.setEnabled(true);
		this.setPenWidthAct.setEnabled(true);
	}

	// /**
	// * FOR TESTING ONLY!
	// */
	// public static void main(String args[])
	// {
	// QApplication.initialize(args);
	// new EditCardGUI(null, new EditCardWindow(new Card())).show();
	//
	// QApplication.exec();
	// }
}
