/**
 * DeckTreeModel.java
 * Group 7, Object Oriented Software Engineering
 * Oct 25, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.util.ArrayList;
import java.util.List;
import com.trolltech.qt.core.QMimeData;
import com.trolltech.qt.core.QModelIndex;
import com.trolltech.qt.core.QObject;
import com.trolltech.qt.core.Qt.DropAction;
import com.trolltech.qt.core.Qt.ItemFlag;
import com.trolltech.qt.core.Qt.ItemFlags;
import com.trolltech.qt.gui.QTreeModel;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.dataModel.Deck;
import edu.jhu.cs.oose.elevel.utilities.ViewState;

/**
 * The model for the deck tree view that provides the data
 * for the tree to populate
 * 
 * @author Pablo Lee
 */
public class DeckTreeModel extends QTreeModel
{
	/** Reference to the ViewState singleton */
	private ViewState state;

	/** Properties of the items in the tree */
	private static final ItemFlags defaultFlags = new ItemFlags(ItemFlag.ItemIsDragEnabled, ItemFlag.ItemIsDropEnabled, ItemFlag.ItemIsSelectable, ItemFlag.ItemIsEnabled);

	/**
	 * Initializes the deck tree using information in {@link ViewState}
	 */
	public DeckTreeModel()
	{
		state = ViewState.INSTANCE;
		state.deckAdded.connect(this, "addDeckAtIndex(Deck, Integer)");
		state.deckRemoved.connect(this, "removeDeckAtIndex(Deck, Integer)");
		state.deckChanged.connect(this, "reloadDeck(Deck)");
		state.cardAdded.connect(this, "addCardAtIndex(Card, Integer)");
		state.cardRemoved.connect(this, "removeCard(Card)");
	}

	/**
	 * Returns the default flags
	 * 
	 * @param index
	 * @return the default flags in the form of the QT ItemFlags object
	 */
	public ItemFlags flags(QModelIndex index)
	{
		return defaultFlags;
	}

	/**
	 * Returns the child specified by <code>index</code>, in the given object
	 * if a deck is passed in a card at index is returned, else deck at index returned
	 * 
	 * @param item
	 *        the deck
	 * @param index
	 *        the index of the card we wish to return
	 * @return the card
	 */
	@Override
	public Object child(Object item, int index)
	{
		if(isDeck(item))
		{
			return ((Deck) item).getCard(index);
		}

		if(!isCard(item))
		{
			return state.getDecks().get(index);
		}
		return null;
	}

	/**
	 * Return the number of children of item, if a deck return num of cards,
	 * if item is not already a deck return the num of decks
	 * 
	 * @param item
	 *        the object with children
	 */
	@Override
	public int childCount(Object item)
	{
		if(isDeck(item))
		{
			return ((Deck) item).numCards();
		}

		if(!isCard(item))
		{
			return state.getDecks().size();
		}
		return 0;
	}

	/**
	 * Returns the text of the object passed in, if a <code>deck</code> object the
	 * name of the <code>deck</code> is return, if a <code>card</code> object the question on the <code>card</code> is
	 * returned
	 * 
	 * @param item
	 *        the object we want to return the text of
	 * @return the text of the object
	 */
	@Override
	public String text(Object item)
	{
		if(isDeck(item))
		{
			return ((Deck) item).getName();
		}
		else if(isCard(item))
		{
			String title = ((Card) item).getTitle();
			if(title.equals(""))
				return "New Card";
			else
				return title;
		}
		return null;
	}

	public final QModelIndex indexOf(Object object)
	{
		if(isDeck(object))
		{
			return index(state.getDecks().indexOf((Deck) object), 0);
		}
		if(isCard(object))
		{
			Deck deck = state.getCurrentDeck();
			QModelIndex deckIndex = indexOf(deck);

			return index(deck.getCards().indexOf((Card) object), 0, deckIndex);
		}
		return null;
	}

	/**
	 * Helper method to check if an object is a <code>Deck</code>
	 * 
	 * @return true is item is a Deck, false otherwise
	 */
	public Boolean isDeck(Object item)
	{
		if(item instanceof Deck)
			return true;
		return false;
	}

	/**
	 * Helper method to check if an object's a {@link Card}
	 * 
	 * @return true is item is a <code>Card</code>, false otherwise
	 */
	public Boolean isCard(Object item)
	{
		return item instanceof Card;
	}

	/**
	 * Adds a deck at the index
	 * 
	 * @param d
	 *        The deck to add
	 * @param index
	 *        The index at which to add it at in the tree
	 */
	@SuppressWarnings("unused")
	private void addDeckAtIndex(Deck d, Integer index)
	{
		childrenInserted(null, index, index);
	}

	/**
	 * Removes the deck at the specified index
	 * 
	 * @param d
	 *        The deck to remove
	 * @param index
	 *        The index from which to remove it at
	 */
	@SuppressWarnings("unused")
	private void removeDeckAtIndex(Deck d, Integer index)
	{
		childrenRemoved(null, index, index);
	}

	/**
	 * Reloads a deck in the tree view by removing its cards and
	 * adding them back again
	 * 
	 * @param deck
	 *        the <code>deck</code> to be updated
	 * @param oldNumCards
	 *        the previous number of cards the <code>deck</code> had
	 */
	@SuppressWarnings("unused")
	private void reloadDeck(Deck d)
	{
		QModelIndex deckIndex = indexOf(d);
		int lastCard = d.numCards() - 1;

		childrenRemoved(deckIndex, 0, lastCard);
		childrenInserted(deckIndex, 0, lastCard);
	}

	/**
	 * Adds the card at the specified index in the
	 * current deck
	 * 
	 * @param c
	 *        The card to add
	 * @param index
	 *        The index at which to add the card
	 *        in the index
	 */
	@SuppressWarnings("unused")
	private void addCardAtIndex(Card c, Integer index)
	{
		QModelIndex deckIndex = indexOf(state.getCurrentDeck());
		childrenInserted(deckIndex, index, index);
	}

	/**
	 * Adds a list of cards to the specified deck at the specified index
	 * 
	 * @param cards
	 *        The list of cards to add
	 * @param deck
	 *        The deck to which to add the cards
	 * @param index
	 *        The index at which to add them at
	 */
	private void addCardsToDeckAtIndex(List<Card> cards, Deck deck, int index)
	{
		QModelIndex deckIndex = indexOf(deck);
		childrenInserted(deckIndex, index, deck.numCards() - 1);
	}

	/**
	 * Removes the card from the current deck
	 * 
	 * @param c
	 *        The card to remove
	 */
	@SuppressWarnings("unused")
	private void removeCard(Card c)
	{
		QModelIndex deckIndex = indexOf(state.getCurrentDeck());
		int cardRow = state.getCurrentDeck().getCards().indexOf(c);
		childrenRemoved(deckIndex, cardRow, cardRow);
	}

	/**
	 * returns the list of string types
	 * 
	 * @return the list of types
	 */
	public List<String> mimeTypes()
	{
		List<String> types = new ArrayList<String>();
		types.add("text/plain");
		return types;
	}

	/**
	 * When setting the mimeData for drag and drop events (mimeData's
	 * how info gets passed on what's been dragged or dropped), use
	 * the new custom mimeData class that also keeps track of what item
	 * is being dragged by overriding this method that returns the mimeData
	 * 
	 * @return the data found in the list or null if not data in list
	 */
	public QMimeData mimeData(List<QModelIndex> list)
	{
		if(list.size() > 0)
		{
			Object item = indexToValue(list.get(0));
			TreeItemMimeData mimeData = new TreeItemMimeData(this);
			mimeData.item = item;
			mimeData.setText(item.toString());
			return mimeData;
		}
		return null;
	}

	/**
	 * The method called after something's been dropped in an item
	 * where objects can be dropped on
	 */
	public boolean dropMimeData(QMimeData data, DropAction action, int row, int col, QModelIndex parentIndex)
	{
		Object receivingItem = indexToValue(parentIndex);

		if(data instanceof TreeItemMimeData)
		{
			Object draggedItem = ((TreeItemMimeData) data).item;

			// If you're moving a card to a new deck
			if(isCard(draggedItem) && isDeck(receivingItem))
			{
				Card movingCard = (Card) draggedItem;
				Deck oldDeck = (Deck) indexToValue(parent(indexOf(movingCard)));
				Deck newDeck = (Deck) receivingItem;

				state.setCurrentCardAndDeck(movingCard, oldDeck);
				state.removeCurrentCard();
				state.setCurrentDeck(newDeck);

				if(row > state.getCurrentDeck().numCards())
				{
					row = state.getCurrentDeck().numCards();
				}

				if(row < 0)
				{
					state.addCard(movingCard);
				}
				else
				{
					state.addCardAtIndex(movingCard, row);
				}
			}

			// If you're moving a deck to a new position
			if(isDeck(draggedItem) && !isDeck(receivingItem) && !isCard(receivingItem))
			{
				if(state.getDecks().size() == 1)
				{
					return false;
				}

				int newPos = row;
				if(row < 0 || row > state.getDecks().size())
				{
					newPos = state.getDecks().size() - 1;
				}

				Deck movingDeck = (Deck) draggedItem;

				state.removeDeck(movingDeck);
				state.addDeckAtIndex(movingDeck, newPos);

			}

			// If you're merging two decks
			if(isDeck(draggedItem) && isDeck(receivingItem))
			{
				Deck movingDeck = (Deck) draggedItem;
				Deck receivingDeck = (Deck) receivingItem;

				if(movingDeck.equals(receivingDeck))
				{
					return false;
				}

				List<Card> cards = movingDeck.getCards();
				state.removeDeck(movingDeck);

				if(cards.size() > 0)
				{
					int index = receivingDeck.numCards();
					receivingDeck.addCards(cards);
					addCardsToDeckAtIndex(cards, receivingDeck, index);
				}
			}
		}

		return false;
	}
}

/**
 * Subclass of QMimeData to also keep track of the object
 * being dragged
 * 
 * @author Pablo Lee
 */
class TreeItemMimeData extends QMimeData
{
	public TreeItemMimeData(QObject object)
	{
	}

	/** The item being dragged */
	public Object item;

	/**
	 * Defines how the Mime data should be printed out when it is an instance of a tree item
	 */
	public String toString()
	{
		return "NodeRefMimeData(" + item.toString() + ")";
	}
}
