/**
 * MainWindow.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import com.trolltech.qt.core.QRect;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QAction;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QDockWidget;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QMainWindow;
import com.trolltech.qt.gui.QMenu;
import com.trolltech.qt.gui.QMenuBar;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QPainter;
import com.trolltech.qt.gui.QPixmap;
import com.trolltech.qt.gui.QPrintDialog;
import com.trolltech.qt.gui.QPrinter;
import com.trolltech.qt.gui.QToolBar;
import com.trolltech.qt.gui.QFileDialog.Filter;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsView;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.EditCardGUI;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.dataModel.CardSection;
import edu.jhu.cs.oose.elevel.dataModel.Deck;
import edu.jhu.cs.oose.elevel.dataModel.TestStat;
import edu.jhu.cs.oose.elevel.editing.EditCardWindow;
import edu.jhu.cs.oose.elevel.graphing.GraphWindow;
import edu.jhu.cs.oose.elevel.graphing.GraphWindowGUI;
import edu.jhu.cs.oose.elevel.preferences.PreferenceWindow;
import edu.jhu.cs.oose.elevel.preferences.Preferences;
import edu.jhu.cs.oose.elevel.testTaking.Test;
import edu.jhu.cs.oose.elevel.testTaking.TestWindowGUI;
import edu.jhu.cs.oose.elevel.utilities.ViewState;

/**
 * Main window that calls the other subunits for the program.
 * 
 * @author H. Parker Shelton, Jesse Yates, Pablo Lee, Greg Anderson
 */
public class MainWindow extends QMainWindow
{
	/** Reference to the state singleton */
	private ViewState state;

	/** Tree widget for the deck of cards */
	private DeckTreeGUI treeView;

	/** Tree widget model */
	private DeckTreeModel treeModel;

	/** Dock widget to hold the tree widget */
	private QDockWidget dockTreeWidget;

	/** The toolbar used for file menu options */
	private QToolBar fileToolbar;

	/** The toolbar used for deck options */
	private QToolBar deckToolbar;

	/** QAction to create a new deck */
	private QAction newDeckAction;

	/** QAction to load a deck */
	private QAction loadDeckAction;

	/** QActions for saving decks */
	private QAction saveDeckAction;
	private QAction saveDeckAsAction;

	/** QActions for printing the current deck */
	private QAction printAction;

	/** The width of the cards printed in inches */
	private final int indexCardWidth = 4;
	private final int indexCardHeight = 2;

	/** QAction for shuffling a deck */
	private QAction shuffleAction;

	/** QAction to display the help prompt */
	private QAction aboutELevelAction;

	/** QAction to display the preference prompt */
	private QAction setPreferencesAction;

	/** QAction to display a graph of statistics */
	private QAction graphAction;

	/** QAction to close the program */
	private QAction exitAction;

	/** Card QActions */
	private QAction newCardAction;
	private QAction editCardAction;
	private QAction duplicateCardAction;
	private QAction testDeckAction;
	private QAction eLvlChallengeDeckAction;

	/** QAction to close decks */
	private QAction closeDeckAction;

	/** QACtion to delete cards */
	private QAction deleteCardAction;

	/** The central card area of the GUI */
	private CardAreaGUI cardArea;

	/**
	 * Initializes a new MainWindow
	 */
	public MainWindow(List<String> open)
	{
		this.state = ViewState.INSTANCE;

		if(Preferences.getInstance().getValue("OPEN_PREVIOUS").equals("true") && open != null && open.size() != 0)
		{
			for(String s : open)
			{
				File f = new File(s);
				if(f.exists())
				{
					try
					{
						Deck d = Deck.readFromDisk(s);
						if(d != null)
							state.addDeck(d);
					}
					catch(FileNotFoundException e)
					{
						e.printStackTrace();
						QMessageBox.critical(this, "FileNotFoundException", "The specified file was not found!");
					}
					catch(IOException e)
					{
						e.printStackTrace();
						QMessageBox.critical(this, "IOException", "An I/O error has occurred!");
					}
					catch(ClassNotFoundException e)
					{
						e.printStackTrace();
						QMessageBox.critical(this, "ClassNotFoundException", "I don't know what a Deck is!");
					}
				}
			}

			if(state.getDecks().size() > 0)
			{
				Deck currentDeck = state.getDecks().get(0);
				state.setCurrentCardAndDeck(currentDeck.getCard(0), currentDeck);
				state.setCurrentDeck(currentDeck);
			}
		}

		initializeActions();
		initializeMenuBar();
		initializeToolBars();
		setupDeckTree();
		setupCardArea();

		state.treeSizeChanged.connect(this, "toggleActions()");
	}

	/**
	 * Initializes the QActions required for MainWindow
	 */
	private void initializeActions()
	{
		this.newDeckAction = new QAction(new QIcon("images/newDeck.png"), tr("Deck"), this);
		this.newDeckAction.setStatusTip(tr("Create a new Deck"));
		this.newDeckAction.setShortcut(tr("Ctrl+N"));
		this.newDeckAction.triggered.connect(this, "newDeck()");

		this.loadDeckAction = new QAction(new QIcon("images/open.png"), tr("Open"), this);
		this.loadDeckAction.setStatusTip(tr("Open Selected Deck"));
		this.loadDeckAction.setShortcut(tr("Ctrl+O"));
		this.loadDeckAction.triggered.connect(this, "openDeck()");

		this.closeDeckAction = new QAction(tr("Close Deck"), this);
		this.closeDeckAction.setStatusTip(tr("Close Selected Deck"));
		this.closeDeckAction.triggered.connect(this, "closeDeck()");

		this.deleteCardAction = new QAction(tr("Delete Card"), this);
		this.deleteCardAction.setStatusTip(tr("Delete Selected Card"));
		this.deleteCardAction.triggered.connect(state, "removeCurrentCard()");

		this.duplicateCardAction = new QAction(tr("Duplicate Card"), this);
		this.duplicateCardAction.setStatusTip(tr("Duplicate the Selected Card"));
		this.duplicateCardAction.triggered.connect(state, "duplicateCard()");

		this.saveDeckAction = new QAction(new QIcon("images/save.png"), tr("Save"), this);
		this.saveDeckAction.setStatusTip(tr("Save Selected Deck"));
		this.saveDeckAction.setShortcut(tr("Ctrl+S"));
		this.saveDeckAction.triggered.connect(this, "saveDeck()");

		this.saveDeckAsAction = new QAction(new QIcon("images/saveAs.png"), tr("Save As"), this);
		this.saveDeckAsAction.setStatusTip(tr("Save Selected Deck As"));
		this.saveDeckAsAction.setShortcut(tr("Ctrl+Shift+S"));
		this.saveDeckAsAction.triggered.connect(this, "saveDeckAs()");

		this.printAction = new QAction(tr("Print"), this);
		this.printAction.setStatusTip(tr("Print Deck"));
		this.printAction.setShortcut(tr("Ctrl+P"));
		this.printAction.triggered.connect(this, "printCurrentDeck()");

		this.shuffleAction = new QAction(new QIcon("images/shuffle.png"), tr("Shuffle"), this);
		this.shuffleAction.setStatusTip(tr("Shuffle Current Deck"));
		this.shuffleAction.triggered.connect(state, "shuffleCurrentDeck()");

		this.aboutELevelAction = new QAction(tr("About E-Level"), this);
		this.aboutELevelAction.setStatusTip(tr("About E-Level"));
		this.aboutELevelAction.triggered.connect(this, "aboutELevel()");

		this.setPreferencesAction = new QAction(tr("Preferences"), this);
		this.setPreferencesAction.setStatusTip(tr("Change Preferences"));
		this.setPreferencesAction.triggered.connect(this, "setPreferences()");

		this.newCardAction = new QAction(new QIcon("images/newCard.png"), tr("Card"), this);
		this.newCardAction.setStatusTip(tr("Create a new card"));
		this.newCardAction.setShortcut(tr("Ctrl+Shift+N"));
		this.newCardAction.triggered.connect(this, "newCard()");

		this.editCardAction = new QAction(new QIcon("images/editCard.png"), tr("Edit Card"), this);
		this.editCardAction.setStatusTip(tr("Edit this Card"));
		this.editCardAction.triggered.connect(this, "editCard()");

		this.testDeckAction = new QAction(new QIcon("images/test.png"), tr("Test"), this);
		this.testDeckAction.setStatusTip(tr("Test this deck"));
		this.testDeckAction.triggered.connect(this, "testCard()");

		this.eLvlChallengeDeckAction = new QAction(new QIcon("images/test.png"), tr("ElvlChallenge"), this);
		this.eLvlChallengeDeckAction.setStatusTip(tr("ELevel Challenge!!"));
		this.eLvlChallengeDeckAction.triggered.connect(this, "eLvlChallenge()");

		this.graphAction = new QAction(tr("Graph Statistics"), this);
		this.graphAction.setStatusTip("Show statistics for this deck");
		this.graphAction.triggered.connect(this, "graphDeck()");

		this.exitAction = new QAction(tr("Exit"), this);
		this.exitAction.setStatusTip(tr("Exit E-Level"));
		this.exitAction.setShortcut(tr("Ctrl+W"));
		this.exitAction.triggered.connect(this, "close()");
	}

	/**
	 * Initializes the menu bar for MainWindow
	 */
	private void initializeMenuBar()
	{
		QMenuBar menuBar = new QMenuBar();
		setMenuBar(menuBar);

		QMenu fileMenu = menuBar.addMenu(tr("&File"));

		QMenu newMenu = fileMenu.addMenu(tr("&New"));
		newMenu.addAction(newDeckAction);
		newMenu.addAction(newCardAction);

		fileMenu.addAction(loadDeckAction);
		fileMenu.addAction(saveDeckAction);
		fileMenu.addAction(saveDeckAsAction);
		fileMenu.addAction(printAction);
		fileMenu.addAction(closeDeckAction);
		fileMenu.addSeparator();
		fileMenu.addAction(exitAction);

		QMenu deckMenu = menuBar.addMenu(tr("&Deck"));
		deckMenu.addAction(testDeckAction);
		deckMenu.addAction(eLvlChallengeDeckAction);
		deckMenu.addAction(shuffleAction);
		deckMenu.addAction(graphAction);

		QMenu cardMenu = menuBar.addMenu(tr("&Card"));
		cardMenu.addAction(editCardAction);
		cardMenu.addAction(deleteCardAction);
		cardMenu.addAction(duplicateCardAction);

		QMenu helpMenu = menuBar.addMenu(tr("&Help"));
		helpMenu.addAction(setPreferencesAction);
		helpMenu.addAction(aboutELevelAction);
	}

	/**
	 * Initializes the basic properties of each the toolbars used
	 */
	private void initializeToolBars()
	{
		this.fileToolbar = addToolBar(tr("File"));
		this.fileToolbar.addAction(newDeckAction);
		this.fileToolbar.addAction(loadDeckAction);
		this.fileToolbar.addAction(saveDeckAction);
		this.fileToolbar.addAction(saveDeckAsAction);

		this.deckToolbar = addToolBar(tr("Deck"));
		this.deckToolbar.addAction(newCardAction);
		this.deckToolbar.addAction(editCardAction);
		this.deckToolbar.addAction(testDeckAction);
		this.deckToolbar.addAction(eLvlChallengeDeckAction);

		if(state.getNumDecks() == 0)
			toggleActions();
	}

	/**
	 * Enables and disables the menu and toolbar items
	 * by checking whether they should be available to the user
	 * or not.
	 */
	private void toggleActions()
	{
		boolean deckEnabled = true;
		boolean cardEnabled = true;

		if(state.getNumDecks() == 0)
		{
			cardEnabled = false;
			deckEnabled = false;
		}
		else if(state.getNumDecks() == 1 && state.getCurrentDeck().numCards() == 0)
		{
			deckEnabled = false;
		}

		editCardAction.setEnabled(deckEnabled);
		testDeckAction.setEnabled(deckEnabled);
		eLvlChallengeDeckAction.setEnabled(deckEnabled);
		shuffleAction.setEnabled(deckEnabled);
		deleteCardAction.setEnabled(deckEnabled);
		duplicateCardAction.setEnabled(deckEnabled);
		testDeckAction.setEnabled(deckEnabled);
		printAction.setEnabled(deckEnabled);
		graphAction.setEnabled(deckEnabled);

		newCardAction.setEnabled(cardEnabled);
		closeDeckAction.setEnabled(cardEnabled);
	}

	/**
	 * Sets up the basic tree view of the {@link Deck}s using the {@link DeckTreeModel}
	 */
	public void setupDeckTree()
	{
		// Create tree view
		treeModel = new DeckTreeModel();
		treeView = new DeckTreeGUI(treeModel);
		treeView.setModel(treeModel);
		if(state.getCurrentDeck() != null)
			treeView.setCurrentIndex(treeModel.valueToIndex(state.getCurrentDeck()));
		treeView.setMinimumWidth(200);

		dockTreeWidget = new QDockWidget(tr("Decks"), this);
		dockTreeWidget.setFeatures(QDockWidget.DockWidgetFeature.NoDockWidgetFeatures);
		dockTreeWidget.setAllowedAreas(Qt.DockWidgetArea.LeftDockWidgetArea);
		dockTreeWidget.setWidget(treeView);
		this.addDockWidget(Qt.DockWidgetArea.LeftDockWidgetArea, dockTreeWidget);
	}

	/**
	 * Sets up the tree view of the {@link Card}s under the currently selected Deck using {@link CardAreaGUI}
	 */
	public void setupCardArea()
	{
		Deck currentDeck = state.getCurrentDeck();
		if(currentDeck != null && currentDeck.numCards() > 0)
			state.setCurrentCardAndDeck(currentDeck.getCard(0), currentDeck);
		this.cardArea = new CardAreaGUI(this);
		setCentralWidget(cardArea);
	}

	/**
	 * Displays an information box about the program
	 */
	@SuppressWarnings("unused")
	private void aboutELevel()
	{
		QMessageBox.about(this, tr("About E-Level"), tr("<b>Iteration 5</b><br><br>" + "<b>Written by:</b><br>H. Parker Shelton - web \"development\", resident Qt \"expert\"<br>" + "Jesse Yates - the wild card<br>" + "Greg Anderson - algorithmic and concept design<br>" + "Pablo Lee - misc.<br>" + "Elaine Zapata - (GUI and group) beautification, model and code review<br>"));
	}

	/**
	 * Creates a new deck
	 */
	@SuppressWarnings("unused")
	private void newDeck()
	{
		DeckDialog open = new DeckDialog(this);
		open.setWindowTitle("New Deck");
		open.setWindowIcon(this.windowIcon());
		open.closing.connect(state, "addDeck(Deck)");
		open.show();
	}

	/**
	 * Opens <code>Deck</code> from disk, prompting the user to select the deck on disk
	 */
	@SuppressWarnings("unused")
	private void openDeck()
	{
		String filepath = QFileDialog.getOpenFileName(this, tr("Open Deck"), (String) Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION"), new Filter("*.elvl"));
		if(filepath == null || filepath.equals(""))
			return;
		try
		{
			Deck deck = Deck.readFromDisk(filepath);
			if(!state.getDecks().contains(deck))
				state.addDeck(deck);
		}
		catch(FileNotFoundException e)
		{
			e.printStackTrace();
			QMessageBox.critical(this, "FileNotFoundException", "The specified file was not found!");
		}
		catch(IOException e)
		{
			e.printStackTrace();
			QMessageBox.critical(this, "IOException", "An I/O error has occurred!");
		}
		catch(ClassNotFoundException e)
		{
			e.printStackTrace();
			QMessageBox.critical(this, "ClassNotFoundException", "I don't know what a Deck is!");
		}
	}

	/**
	 * Save the <code>deck</code> based on the deck's current location on disk
	 */
	@SuppressWarnings("unused")
	private void saveDeck() throws IOException
	{
		String filepath = state.getCurrentDeck().getDiskLocation();
		if(filepath == null || filepath.equals(""))
			saveDeckAs();
		else
		{
			try
			{
				state.getCurrentDeck().writeToDisk(filepath);
			}
			catch(IOException e)
			{
				QMessageBox.critical(this, "IOException", "An I/O error has occurred!");
				e.printStackTrace();
			}
		}

	}

	/**
	 * Save the <code>Deck</code> by prompting the user for a location on disk
	 */
	private void saveDeckAs()
	{
		String filepath = QFileDialog.getSaveFileName(this, tr("Save Deck"), (String) Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION"), new Filter("*.elvl"));
		if(filepath == null || filepath.equals(""))
			return;
		try
		{
			state.getCurrentDeck().writeToDisk(filepath);
		}
		catch(IOException e)
		{
			QMessageBox.critical(this, "IOException", "An I/O error has occurred!");
			e.printStackTrace();
		}
	}

	/**
	 * Print the current deck such that you can do
	 * double sided printing to get questions on one side
	 * and answers on the other.
	 */

	@SuppressWarnings("unused")
	private void printCurrentDeck()
	{
		QPrinter printer = new QPrinter();
		printer.setPageMargins(1, 1, 1, 1, QPrinter.Unit.Inch);
		printer.setDuplex(QPrinter.DuplexMode.DuplexShortSide);
		printer.setDoubleSidedPrinting(true);
		printer.setOrientation(QPrinter.Orientation.Landscape);
		QPrintDialog printDialog = new QPrintDialog(printer, this);
		if(printDialog.exec() == QDialog.DialogCode.Accepted.value())
		{
			List<Card> cards = state.getCurrentDeck().getCards();
			int cardWidth = indexCardWidth * printer.logicalDpiX();
			int cardHeight = indexCardHeight * printer.logicalDpiY();
			QRect pageBounds = printer.pageRect();
			int numCardsHoriz = pageBounds.width() / cardWidth;
			int numCardsVert = pageBounds.height() / cardHeight;
			int numCardsPerPage = numCardsHoriz * numCardsVert;
			int numPages = (int) Math.ceil(cards.size() / (double) numCardsPerPage);
			// We want to center the cards in the page
			int startingXPos = (pageBounds.width() - numCardsHoriz * cardWidth) / 2;
			QPainter painter = new QPainter();
			painter.begin(printer);
			int x = startingXPos;
			int y = 0;
			int cardNum = 0;
			String text = "";
			QPixmap image = null;
			for(int page = 0; page < numPages; page++)
			{
				// For both the page with questions, and the
				// page with answers
				for(int face = 0; face < 2; face++)
				{
					y = 0;
					for(int j = 0; j < numCardsVert; j++)
					{
						x = startingXPos;
						for(int i = 0; i < numCardsHoriz; i++)
						{
							int cardIndex = cardNum + j * numCardsHoriz + i;
							// If it's the answers, we want to revert the order
							// in which they're shown, so that when you do double
							// sided printing, they're behind their corresponding
							// questions.
							if(face == 1)
							{
								cardIndex = cardNum + j * numCardsHoriz + (numCardsHoriz - 1 - i);
							}
							if(cardIndex >= cards.size())
							{
								text = "";
							}
							else if(face == 0)
							{
								CardSection question = cards.get(cardIndex).getQuestion();
								if(question.hasText())
									text = question.getText();
								else
									image = new DrawingGraphicsView(question.getScene()).getImage();
							}
							else
							{
								CardSection answer = cards.get(cardIndex).getAnswer();
								if(answer.hasText())
									text = answer.getText();
								else
									image = new DrawingGraphicsView(answer.getScene()).getImage();
							}
							QRect cardRectangle = new QRect(x, y, cardWidth, cardHeight);
							QRect textRectangle = new QRect(x + 5, y + 5, cardWidth - 10, cardHeight - 10);
							painter.drawRect(cardRectangle);
							if(text.equals("") && image != null)
								painter.drawImage(textRectangle, image.toImage());
							else
								painter.drawText(textRectangle, (Qt.AlignmentFlag.AlignCenter.value() | Qt.TextFlag.TextWordWrap.value()), text);
							x += cardWidth;
						}
						y += cardHeight;
					}
					if(page != numPages - 1 || face == 0)
					{
						printer.newPage();
					}
				}
				cardNum += numCardsPerPage;
			}
			painter.end();
		}
	}

	/**
	 * Display the settings that can be changed
	 */
	@SuppressWarnings("unused")
	private void setPreferences()
	{
		PreferenceWindow preferences = new PreferenceWindow(this);
		preferences.exec();
	}

	/**
	 * Displays an {@link EditCardGUI} populated with a blank card
	 */
	@SuppressWarnings("unused")
	private void newCard()
	{
		state.addCard(new Card());
		editCard();
	}

	/**
	 * Closes a deck by removing it from the tree
	 */
	@SuppressWarnings("unused")
	private void closeDeck()
	{
		Deck d = state.getCurrentDeck();
		if(d.hasChanged())
			if(!displaySaveDeckPrompt(d))
				return;
		state.removeDeck(d);
	}

	/**
	 * Displays an {@link EditCardGUI} populated with the currently selected card
	 */
	private void editCard()
	{
		if(state.getCurrentDeck() == null)
			state.setCurrentDeck(new Deck("Temporary"));
		Card c = state.getCurrentCard();

		EditCardWindow edit = new EditCardWindow(c);
		// HPS - Advanced Card Edit
		EditCardGUI editGUI = new EditCardGUI(this, edit);// , flags);
		editGUI.setWindowIcon(this.windowIcon());
		editGUI.setWindowTitle(tr("Edit Card"));
		editGUI.closing.connect(state, "refreshCurrentCard()");
		QDesktopWidget qdw = new QDesktopWidget();
		int screenCenterX = qdw.width() / 2;
		int screenCenterY = qdw.height() / 2;
		editGUI.setGeometry(screenCenterX - 400, screenCenterY - 300, 800, 600);
		editGUI.show();// FullScreen();
	}

	/**
	 * Displays the {@link TestWindowGUI}
	 */
	@SuppressWarnings("unused")
	private void testCard()
	{
		HashMap<String, String> preferences = new HashMap<String, String>();
		if(Preferences.getInstance().getValue("SHOW_TEST_PREFERENCES").equals("true"))
			preferences = PreferenceWindow.showTestPreferences(this);

		if(preferences == null)
			return;

		Test test = new Test(state.getCurrentDeck(), preferences);

		TestWindowGUI gui = new TestWindowGUI(test, this);
		gui.setWindowTitle("Testing");
		gui.show();// FullScreen();
		this.setFocus(Qt.FocusReason.ActiveWindowFocusReason);
	}

	/**
	 * Starts the E-Level Challenge, a challenge
	 * where all the incorrect questions from the past
	 * week are asked again.
	 */
	@SuppressWarnings("unused")
	private void eLvlChallenge()
	{
		HashMap<String, String> preferences = new HashMap<String, String>();
		if(Preferences.getInstance().getValue("SHOW_TEST_PREFERENCES").equals("true"))
			preferences = PreferenceWindow.showTestPreferences(this);

		if(preferences == null)
			return;

		Deck deck = getELevelChallengeDeck();
		if(deck.numCards() == 0)
		{
			QMessageBox box = new QMessageBox();
			box.setText("Error: Could not generate E Level Challenge Deck. Make sure you have a deck open with which you have taken a test (and gotten some incorrect answers).");
			box.exec();
			return;
		}
		Test test = new Test(deck, preferences);

		TestWindowGUI gui = new TestWindowGUI(test, this);
		gui.setWindowTitle("E Level Challenge");
		gui.show();// FullScreen();
		this.setFocus(Qt.FocusReason.ActiveWindowFocusReason);
	}

	/**
	 * Returns a map of cards to probabilities for the ELevel Challenge quiz.
	 * The higher the number of times a deck was incorrectly answered the higher the probability in the map.
	 * 
	 * @return a <code>Map<Card, Float></code> of <code>Card</code>s and their associated probability
	 */
	private Deck getELevelChallengeDeck()
	{
		Deck elvlDeck = new Deck();

		Date currentDate = new Date(System.currentTimeMillis());
		Date previousDate = new Date(System.currentTimeMillis() - 604800000);

		for(Deck deck : state.getDecks())
		{
			for(Card card : deck.getCards())
			{
				for(TestStat stat : deck.getTestStatistics())
				{
					// if the statistic set is over 7 days old we ignore it.
					if(currentDate.compareTo(stat.getDate()) >= 0 && previousDate.compareTo(stat.getDate()) <= 0)
					{
						if(stat.getIncorrectlyAnsweredCards().contains(card.getID()) && !elvlDeck.getCards().contains(card))
						{
							// XXX:might need to do a better check to see if elvl already contains the card.
							// alternatively: have the "incorrectly answered card list" be a list of cards rather than
							// strings.
							// this would be greatly preferred.
							elvlDeck.addCard(card);
						}
					}
				}
			}
		}

		return elvlDeck;
	}

	/**
	 * Handles closing actions for this window. Prompts to save any <code>Deck</code> that has changed
	 * and writes preferences out to disk.
	 * 
	 * @see com.trolltech.qt.gui.QWidget#closeEvent(com.trolltech.qt.gui.QCloseEvent)
	 */
	protected void closeEvent(QCloseEvent e)
	{
		if(Preferences.getInstance().getValue("OPEN_PREVIOUS").equals("true"))
			Preferences.getInstance().writeSettings(state.getDecks());
		else
			Preferences.getInstance().writeSettings();

		for(Deck d : state.getDecks())
		{
			if(d.hasChanged())
			{
				boolean b = displaySaveDeckPrompt(d);
				if(!b)
				{
					e.ignore();
					return;
				}
				else
					e.accept();
			}
		}
	}

	protected void graphDeck()
	{
		Deck d = state.getCurrentDeck();
		if(d == null)
			return;

		GraphWindow graph = new GraphWindow(d);
		GraphWindowGUI gui = new GraphWindowGUI(graph);
		gui.show();
		this.setFocus(Qt.FocusReason.ActiveWindowFocusReason);
	}

	/**
	 * Displays a prompt asking to save the specified Deck
	 * 
	 * @param d
	 *        a <code>Deck</code> that has changed
	 * @return <code>false</code> if the close event has been canceled, <code>true</code> otherwise
	 */
	private boolean displaySaveDeckPrompt(Deck d)
	{
		QMessageBox.StandardButton r = QMessageBox.warning(this, tr("Edit Card"), "Deck \"" + d.getName() + "\" has been modified.\n" + "Save your changes?", new QMessageBox.StandardButtons(QMessageBox.StandardButton.Yes, QMessageBox.StandardButton.No, QMessageBox.StandardButton.Cancel));
		if(r == QMessageBox.StandardButton.Yes)
		{
			try
			{
				state.getCurrentDeck().writeToDisk(d.getDiskLocation());
			}
			catch(IOException e)
			{
				QMessageBox.critical(this, "IOException", "An I/O error has occurred!");
				e.printStackTrace();
			}
			return true;
		}
		if(r == QMessageBox.StandardButton.Cancel)
			return false;
		return true;
	}
}
