/**
 * Card.java
 * Group 7, Object Oriented Software Engineering
 * November 21, 2009
 */
package edu.jhu.cs.oose.elevel.dataModel;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;

/**
 * <code>Card</code> represents the high-level intuition of a flash card. <code>Card</code> represents a single card in
 * a {@link Deck} and maintains all the features of a real flash-card as well as additional, computer-only
 * possibilities.
 * 
 * @author Jesse Yates, H. Parker Shelton
 */
public class Card implements Serializable
{
	private static final long serialVersionUID = 67658268L;

	/** A <code>String</code> representing the title of the card */
	protected String title;

	/** A {@link CardSection} representing the question associated with this <code>Card</code> */
	protected CardSection question;

	/** A <code>CardSection</code> representing the answer associated with this <code>Card</code> */
	protected CardSection answer;

	/** A <code>CardSection</code> representing a hint associated with this <code>Card</code> */
	protected CardSection hint;

	/** A <code>CardSection</code> representing additional information associated with this </code>Card</code> */
	protected CardSection additionalInfo;

	/**
	 * An <code>int</code> representing a unique identifier for this card. Note: this field is "package protected"
	 * (when no explicit modifier is given). This is the desired behavior for this field. We only want Card, Deck, and
	 * TestStat to be able to modify a card's id.
	 **/
	int ID;

	/** A {@link Map} representing the card's options */
	protected Map<String, Boolean> options;

	/**
	 * Instantiates a <code>Card</code>
	 */
	public Card()
	{
		this.question = new CardSection();
		this.answer = new CardSection();
		this.hint = new CardSection();
		this.additionalInfo = new CardSection();
		this.title = "";

		this.options = new HashMap<String, Boolean>();
		this.ID = -1;

		setDefaultSettings();
	}

	/**
	 * Instantiates a <code>Card</code> with the specified <code>CardSection</code> parameters.
	 * 
	 * @param q
	 *        a <code>CardSection</code> representing the question associated with this <code>Card</code>
	 * @param a
	 *        a <code>CardSection</code> representing the answer associated with this <code>Card</code>
	 * @param h
	 *        a <code>CardSection</code> representing the hint associated with this <code>Card</code>
	 * @param i
	 *        a <code>CardSection</code> representing the additional information associated with this <code>Card</code>
	 */
	public Card(CardSection q, CardSection a, CardSection h, CardSection i)
	{
		this();

		this.question = q;
		this.answer = a;
		this.hint = h;
		this.additionalInfo = i;
	}

	/**
	 * Instantiates a <code>Card</code> with the specified <code>DrawingGraphicsScene</code> parameters.
	 * 
	 * @param q
	 *        a <code>DrawingGraphicsScene</code> representing the question associated with this <code>Card</code>
	 * @param a
	 *        a <code>DrawingGraphicsScene</code> representing the answer associated with this <code>Card</code>
	 * @param h
	 *        a <code>DrawingGraphicsScene</code> representing the hint associated with this <code>Card</code>
	 * @param i
	 *        a <code>DrawingGraphicsScene</code> representing the additional information associated with this
	 *        <code>Card</code>
	 */
	public Card(DrawingGraphicsScene q, DrawingGraphicsScene a, DrawingGraphicsScene h, DrawingGraphicsScene i)
	{
		this();

		this.question.setScene(q);
		this.answer.setScene(a);
		this.hint.setScene(h);
		this.additionalInfo.setScene(i);
	}

	/**
	 * Instantiates a <code>Card</code> with the specified <code>CardSection</code> parameters.
	 * 
	 * @param t
	 *        a <code>String</code> representing the title of this <code>Card</code>
	 * @param q
	 *        a <code>CardSection</code> representing the question associated with this <code>Card</code>
	 * @param a
	 *        a <code>CardSection</code> representing the answer associated with this <code>Card</code>
	 * @param h
	 *        a <code>CardSection</code> representing the hint associated with this <code>Card</code>
	 * @param i
	 *        a <code>CardSection</code> representing the additional information associated with this <code>Card</code>
	 */
	public Card(String t, CardSection q, CardSection a, CardSection h, CardSection i)
	{
		this(q, a, h, i);
		this.title = t;
	}

	/**
	 * Returns whether the contents of this <code>Card</code> equals the contents of another <code>Card</code>. Two
	 * <code>Card</code>s that satisfy equals() may be distinct cards in the same or different <code>Deck</code>s, but
	 * have identical contents.
	 * 
	 * @param other
	 *        the <code>Card</code> to be compared to this <code>Card</code>
	 * @return <code>true</code> if the contents of the <code>Card</code>s are the same, <code>false</code> otherwise
	 */
	public boolean equals(Card other)
	{
		return this.hashCode() - this.ID == other.hashCode() - other.getID();
	}

	/**
	 * Returns the additional information associated with this card
	 * 
	 * @return a <code>CardSection</code> representing the additional information associated with this <code>Card</code>
	 */
	public CardSection getAdditionalInfo()
	{
		return this.additionalInfo;
	}

	/**
	 * Returns the answer associated with this <code>Card</code>
	 * 
	 * @return a <code>CardSection</code> representing this <code>Card</code>'s answer
	 */
	public CardSection getAnswer()
	{
		return this.answer;
	}

	/**
	 * Returns the hint associated with this card
	 * 
	 * @return a <code>CardSection</code> representing the hint associated with this <code>Card</code>
	 */
	public CardSection getHint()
	{
		return this.hint;
	}

	/**
	 * Returns the <code>Card</code>'s deck-dependent ID.
	 * 
	 * @return an <code>int</code> representing the <code>Card</code>'s deck-dependent ID.
	 */
	public int getID()
	{
		return this.ID;
	}

	/**
	 * Returns the <code>Card</code>'s options
	 * 
	 * @return a <code>Map<String, Boolean></code> representing enabled and disabled options
	 */
	public Map<String, Boolean> getOptions()
	{
		return this.options;
	}

	/**
	 * Returns the question associated with this <code>Card</code>
	 * 
	 * @return a <code>CardSection</code> representing this <code>Card</code>'s question
	 */
	public CardSection getQuestion()
	{
		return this.question;
	}

	/**
	 * Returns the title associated with this card
	 * 
	 * @return a <code>String</code> representing the title of this <code>Card</code>
	 */
	public String getTitle()
	{
		if(this.title.equals(""))
			if(this.question.hasText())
				return this.question.getText();
		return this.title;
	}

	/**
	 * Computes the hash code of this card based on its contents as well as the order in which it was added to this
	 * <code>Deck</code>. Cards with the same hashCodes are exactly the same <code>Card</code>.
	 * 
	 * @return an <code>int</code> hash code
	 */
	public int hashCode()
	{
		return (this.question != null ? 97 * this.question.hashCode() : 0) + (this.answer != null ? 79 * this.answer.hashCode() : 0) + (this.hint != null ? 61 * this.hint.hashCode() : 0) + (this.additionalInfo != null ? 47 * this.additionalInfo.hashCode() : 0) + (this.options != null ? 3 * this.options.hashCode() : 0) + this.ID;
	}

	/**
	 * Sets the additional information associated with this <code>Card</code> to the specified
	 * <code>DrawingGraphicsScene</code>
	 * 
	 * @param i
	 *        a <code>DrawingGraphicsScene<code> representing the <code>Card</code>'s additional information
	 */
	public void setAdditionalInfo(DrawingGraphicsScene i)
	{
		this.additionalInfo.setScene(i);
	}

	/**
	 * Sets the additional information associated with this <code>Card</code> to the specified <code>String</code>
	 * 
	 * @param i
	 *        a <code>String<code> representing the <code>Card</code>'s additional information
	 */
	public void setAdditionalInfo(String i)
	{
		this.additionalInfo = new CardSection(i);
	}

	/**
	 * Sets this <code>Card</code>'s to the specified <code>DrawingGraphicsScence</code>
	 * 
	 * @param a
	 *        a <code>DrawingGraphicsScence<code> representing the <code>Card</code>'s answer
	 */
	public void setAnswer(DrawingGraphicsScene a)
	{
		this.answer.setScene(a);
	}

	/**
	 * Set the default <code>Card</code> settings
	 */
	private void setDefaultSettings()
	{
		this.options.clear();
		this.options.put("Display Hints", new Boolean(false));
		this.options.put("Self-check Answer", new Boolean(false));
	}

	/**
	 * Sets the hint associated with this card to the specified <code>DrawingGraphicsScene</code>
	 * 
	 * @param h
	 *        a <code>DrawingGraphicsScene</code> representing the <code>Card</code>'s hint
	 */
	public void setHint(DrawingGraphicsScene h)
	{
		this.hint.setScene(h);
	}

	/**
	 * Sets the hint associated with this card to the specified <code>String</code>
	 * 
	 * @param h
	 *        a <code>String</code> representing the <code>Card</code>'s hint
	 */
	public void setHint(String h)
	{
		this.hint = new CardSection(h);
	}

	/**
	 * Sets the <code>Card</code>'s options to the specified options
	 * 
	 * @param p
	 *        A <code>Map<String, Boolean> representing enabled and disabled options
	 */
	public void setOptions(Map<String, Boolean> p)
	{
		if(p != null)
		{
			this.options.clear();
			this.options.putAll(p);
		}
	}

	/**
	 * Sets the question associated with this card to the specified <code>DrawingGraphicsScene</code>
	 * 
	 * @param q
	 *        a <code>DrawingGraphicsScene<code> representing the <code>Card</code>'s question
	 */
	public void setQuestion(DrawingGraphicsScene q)
	{
		this.question.setScene(q);
	}

	/**
	 * Sets the specified setting, updating the setting if it already exists
	 * 
	 * @param key
	 *        a <code>String</code> representing the name of the setting
	 * @param value
	 *        the new boolean value of the setting
	 */
	@SuppressWarnings("unused")
	private void setSetting(String key, Boolean value)
	{
		if(this.options != null)
			this.options.put(key, value);
	}

	/**
	 * Sets the title associated with this card to the specified <code>String</code>
	 * 
	 * @param t
	 *        a <code>String</code> representing the <code>Card</code>'s title
	 */
	public void setTitle(String t)
	{
		if(t != null)
			this.title = t;
	}

	/**
	 * Set the values of this <code>Card</code> to the specified <code>DrawingGraphicsScene</code>s
	 * 
	 * @param q
	 *        a <code>DrawingGraphicsScene</code> representing the question associated with this <code>Card</code>
	 * @param a
	 *        a <code>DrawingGraphicsScene</code> representing the answer associated with this <code>Card</code>
	 * @param h
	 *        a <code>DrawingGraphicsScene</code> representing the hint associated with this <code>Card</code>
	 * @param i
	 *        a <code>DrawingGraphicsScene</code> representing the additional information associated with this
	 *        <code>Card</code>
	 * @param pref
	 *        A <code>Map<String, Boolean></code> representing the options associated with this <code>Card</code>
	 */
	public void setValues(DrawingGraphicsScene q, DrawingGraphicsScene a, DrawingGraphicsScene h, DrawingGraphicsScene i, Map<String, Boolean> pref)
	{
		this.question.setScene(q);
		this.answer.setScene(a);
		this.hint.setScene(h);
		this.additionalInfo.setScene(i);
		setOptions(pref);
	}

	/**
	 * Returns a <code>String</code> representation of this <code>Card</code>
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		StringBuffer b = new StringBuffer();
		b.append("-----\n");
		b.append("Card:\n");
		b.append("Question: " + this.question.getScene().toString() + "\n");
		b.append("Answer: " + this.answer.getScene().toString() + "\n");
		b.append("Additional Info: " + this.additionalInfo.getScene().toString() + "\n");
		b.append("Hint: " + this.hint.getScene().toString() + "\n");
		b.append("Settings: \n");

		for(Entry<String, Boolean> i : this.options.entrySet())
			b.append(i.getKey() + ":" + i.getValue().toString() + "\n");
		b.append("-----\n");
		return b.toString();
	}
}
