/**
 * CardSection.java
 * Group 7, Object Oriented Software Engineering
 * November 22, 2009
 */
package edu.jhu.cs.oose.elevel.dataModel;

import java.io.Serializable;
import java.util.List;
import com.trolltech.qt.gui.QGraphicsItemInterface;
import com.trolltech.qt.gui.QGraphicsTextItem;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;

/**
 * <code>CardSection</code> represents a particular section of a flash card, for example the question or answer. A
 * <code>CardSection</code> may either contain text or multimedia.
 * 
 * @author H. Parker Shelton
 */
public class CardSection implements Serializable
{
	private static final long serialVersionUID = 83696784737978L;

	/** The size of all <code>CardSection</code>s */
	public static final int WIDTH = 400;
	public static final int HEIGHT = 240;

	/** A {@link DrawingGraphicsScene} containing the multimedia in this <code>CardSection</code> */
	private DrawingGraphicsScene scene;

	/**
	 * A <code>String</code> representation of the contents of this <code>CardSection</code>,
	 * set if the only object is a text box
	 */
	private String text;

	/** <code>true</code> if the <code>CardSection</code> has text */
	private boolean hasText;

	/**
	 * Instantiates a <code>CardSection</code>
	 */
	public CardSection()
	{
		this.hasText = false;
		this.text = "";
		this.scene = new DrawingGraphicsScene();
	}

	/**
	 * Instantiates a <code>CardSection</code> with the specified <code>DrawingGraphicsScene</code>
	 * 
	 * @param s
	 *        a <code>DrawingGraphicsScene</code> representing the multimedia contents of this <code>CardSection</code>
	 */
	public CardSection(DrawingGraphicsScene s)
	{
		this.hasText = false;
		this.text = "";
		this.scene = s;
	}

	/**
	 * Instantiates a <code>CardSection</code> with the specified <code>String</code>
	 * 
	 * @param s
	 *        a <code>String</code> representing the text contents of this <code>CardSection</code>
	 */
	public CardSection(String s)
	{
		if(s != null && !s.equals(""))
		{
			this.hasText = true;
			this.text = s;
		}
		this.scene = new DrawingGraphicsScene();
	}

	/**
	 * Returns a copy of this <code>CardSection</code>
	 */
	public CardSection clone()
	{
		CardSection clone = new CardSection(this.text);
		clone.setScene(this.scene.clone());
		return clone;
	}

	/**
	 * Returns <code>true</code> if the both <code>CardSection</code>s have the same contents, <code>false</code>
	 * otherwise
	 * 
	 * @param other
	 *        a <code>CardSection</code> for comparison
	 * @return <code>true</code> if the both <code>CardSection</code>s have the same contents, <code>false</code>
	 *         otherwise
	 */
	public boolean equals(CardSection other)
	{
		if(this.hasText != other.hasText())
			return false;
		if(this.hasText)
			return this.text.equals(other.getText());
		return this.scene.hashCode() == other.getScene().hashCode();
	}

	/**
	 * Returns the <code>CardSection</code>'s associated <code>DrawingGraphicsScene</code>
	 * 
	 * @return the <code>CardSection</code>'s associated <code>DrawingGraphicsScene</code>
	 */
	public DrawingGraphicsScene getScene()
	{
		return this.scene;
	}

	/**
	 * Returns the <code>CardSection</code>'s associated <code>String</code> contents
	 * 
	 * @return the <code>CardSection</code>'s associated <code>String</code> contents
	 */
	public String getText()
	{
		if(this.hasText)
			return this.text;
		return "";
	}

	/**
	 * Generates an <code>int<code> hashcode for this <code>CardSection</code>
	 * 
	 * @return an <code>int<code> hashcode for this <code>CardSection</code>
	 */
	public int hashCode()
	{
		if(this.hasText)
			return this.text != null ? this.text.hashCode() : 0;
		return this.scene != null ? this.scene.hashCode() : 0;
	}

	/**
	 * Returns <code>true</code> if this <code>CardSection</code> has text content
	 * 
	 * @return <code>true</code> if this <code>CardSection</code> has text content
	 */
	public boolean hasText()
	{
		return this.hasText;
	}

	/**
	 * Sets the associated <code>DrawingGraphicsScene</code> to the specified <code>DrawingGraphicsScene</code>
	 * 
	 * @param s
	 *        the new <code>DrawingGraphicsScene</code>
	 */
	public void setScene(DrawingGraphicsScene s)
	{
		if(s != null)
		{
			this.scene = s;
			List<QGraphicsItemInterface> items = s.items();
			if(items.size() == 1 && items.get(0) instanceof QGraphicsTextItem)
			{
				this.text = ((QGraphicsTextItem) items.get(0)).toPlainText();
				this.hasText = true;
			}
			else
			{
				this.text = "";
				this.hasText = false;
			}
		}
		else
		{
			this.scene = new DrawingGraphicsScene();
			this.text = "";
			this.hasText = false;
		}
	}

	/**
	 * Return a <code>String</code> representation of this <code>CardSection</code>
	 */
	public String toString()
	{
		if(this.hasText)
			return this.text;
		else
			return this.scene.toString();
	}
}
