/**
 * Deck.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel.dataModel;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

/**
 * {@link Deck} represents the high-level intuition of a deck of flash cards.
 * The class stores a set of {@link Card}s and set of {@link TestStat}s based on past tests.
 * 
 * @author Jesse Yates, Greg Anderson, H. Parker Shelton
 */
public class Deck implements Serializable
{
	private static final long serialVersionUID = 68696775L;

	/**
	 * Read a <code>Deck</code> from disk given an absolute file name
	 * 
	 * @param filepath
	 *        A <code>String</code> representing the absolute path file name of the file
	 * @return a {@link Deck} created from the specified file
	 */
	public static Deck readFromDisk(String filepath) throws FileNotFoundException, IOException, ClassNotFoundException
	{
		ObjectInputStream in = new ObjectInputStream(new FileInputStream(filepath));
		Deck d = (Deck) in.readObject();
		in.close();

		if(d != null)
		{
			d.setDiskLocation(filepath);
			d.setHasChanged(false);
		}
		return d;
	}

	/** The location of this <code>Deck</code> on disk */
	private String diskLocation;

	/** The name of this <code>Deck</code> */
	private String name;

	/** The set of statistics for this <code>Deck</code>, compiled over time */
	private List<TestStat> testStats;

	/** The set of cards contained in this <code>Deck</code> */
	private List<Card> cards;

	/** The value used for the id of the next card to be added to this deck. Incremented automatically. */
	private int cardIdIterator;

	/** A boolean indicating whether the given Deck has been changed */
	private boolean hasChanged;

	public Deck()
	{
		this("");
	}

	/**
	 * Initialize a basic <code>Deck</code> using the name provided
	 * 
	 * @param name
	 *        A String representing the name of the <code>Deck</code>
	 */
	public Deck(String name)
	{
		setName(name);
		this.cardIdIterator = 0;
		this.hasChanged = false;
		this.cards = new ArrayList<Card>();
		this.testStats = new ArrayList<TestStat>();
	}

	/**
	 * Add the specified {@link Card} to the <code>Deck</code>
	 * 
	 * @param c
	 *        a {@link Card} to be added to the deck
	 */
	public boolean addCard(Card c)
	{
		this.hasChanged = true;

		if(c.getID() == -1)
			c.ID = getUnusedID();

		return this.cards.add(c);
	}

	/**
	 * Add the specified {@link Card} to the <code>Deck</code> at position <code>i</code>
	 * 
	 * @param i
	 *        An <code>int</code> representing the position at which to add the specified card
	 * @param c
	 *        The <code>Card</code> to be added
	 */
	public void addCardAtIndex(int i, Card c)
	{
		this.hasChanged = true;
		if(c.getID() == -1)
			c.ID = getUnusedID();
		this.cards.add(i, c);
	}

	/**
	 * Add the list of cards to this deck
	 * 
	 * @param cards
	 *        A <code>List</code> of cards to add to this deck
	 * @return <code>true</code> if all cards were successfully added to the deck, <code>false</code> otherwise
	 */
	public boolean addCards(List<Card> cards)
	{
		this.hasChanged = true;
		for(Card card : cards)
			if(card.getID() == -1)
				card.ID = getUnusedID();
		this.cards.addAll(cards);
		return true;
	}

	/**
	 * Adds a new testing statistic to the current set of statistics for the deck
	 * 
	 * @param d
	 *        {@link TestStat} to be associated with the deck
	 * @return <code>true</code> if the {@link TestStat} is inserted, <code>false</code> otherwise
	 */
	public boolean addTestStat(TestStat d)
	{
		this.hasChanged = true;
		return this.testStats.add(d);
	}

	/**
	 * Removes all {@link Card}s from the <code>Deck</code>
	 * 
	 * @see java.util.Collection#clear()
	 */
	public void clearCards()
	{
		this.hasChanged = true;
		this.cards.clear();
	}

	/**
	 * Clears all of the statistics associated with this <code>Deck</code>
	 * 
	 * @see java.util.Collection#clear()
	 */
	public void clearTestStats()
	{
		this.hasChanged = true;
		this.testStats.clear();
	}

	/**
	 * Returns <code>true</code> if this <code>Deck</code> contains all specified {@link Card}s
	 * 
	 * @param cards
	 *        A <code>Collection<code> to be checked for containment in this collection
	 * @see java.util.Collection#containsAll(Collection)
	 */
	public boolean containsAllCards(Collection<Card> cards)
	{
		return this.cards.containsAll(cards);
	}

	/**
	 * Returns <code>true</code> if this <code>Deck</code> contains the specified {@link Card}
	 * 
	 * @param c
	 *        the {@link Card} whose presence in this <code>Deck</code> is to be tested
	 * @see java.util.Collection#contains(Object)
	 */
	public boolean containsCard(Card c)
	{
		return this.cards.contains(c);
	}

	public boolean equals(Deck other)
	{
		return this.hashCode() == other.hashCode();
	}

	/**
	 * Returns the {@link Card} at the index of this deck
	 * 
	 * @param index
	 * @return The card at the index of this deck
	 */
	public Card getCard(int index)
	{
		return this.cards.get(index);
	}

	/**
	 * Returns the {@link Collection} of {@link Card}s in this <code>Deck</code>
	 * 
	 * @return the {@link Collection} of {@link Card}s in this <code>Deck</code>
	 */
	public List<Card> getCards()
	{
		return this.cards;
	}

	/**
	 * Returns a {@code String} representing the location of this {@code Deck} on disk
	 * 
	 * @return a {@code String} representing the location of this {@code Deck} on disk
	 */
	public String getDiskLocation()
	{
		return this.diskLocation;
	}

	/**
	 * Helper method that returns the first card in the deck
	 * 
	 * @return First card of the deck
	 */
	public Card getFirstCard()
	{
		return this.cards.get(0);
	}

	/**
	 * Helper method that returns the last card in the deck
	 * 
	 * @return The last card in the deck
	 */
	public Card getLastCard()
	{
		return this.cards.get(numCards() - 1);
	}

	/**
	 * Returns a <code>String</code> representing the name of this <code>Deck</code>
	 * 
	 * @return a <code>String</code> representing the name of this <code>Deck</code>
	 */
	public String getName()
	{
		return this.name;
	}

	/**
	 * Returns a {@link Collection} of {@link TestStat}s representing the cumulative statistics of this deck
	 * 
	 * @return a {@link Collection} of {@link TestStat}s representing the cumulative statistics of this deck
	 */
	public Collection<TestStat> getTestStatistics()
	{
		return this.testStats;
	}

	/**
	 * returns the current cardIdIterator value and increments it
	 * 
	 * @return the unused ID
	 */
	private int getUnusedID()
	{
		return ++this.cardIdIterator;
	}

	/**
	 * @return The value of the hasChanged field.
	 */
	public boolean hasChanged()
	{
		return this.hasChanged;
	}

	public int hashCode()
	{
		return (this.name != null ? 97 * this.name.hashCode() : 0) + (this.testStats != null ? 79 * this.testStats.hashCode() : 0) + (this.diskLocation != null ? 61 * this.diskLocation.hashCode() : 0) + (this.cards != null ? 47 * this.cards.hashCode() : 0) + (this.hasChanged ? 1 : 0);
	}

	/**
	 * Returns the index of the specified <code>Card</code> in the specified <code>Deck</code>
	 * 
	 * @return an <code>int</code> representing the index of the specified <code>Card</code> in the specified
	 *         <code>Deck</code>, -1 if the <code>Card</code> is not contained in the specified <code>Deck</code>
	 */
	public int indexOf(Card c)
	{
		return this.cards.indexOf(c);
	}

	/**
	 * Returns <code>true</code> if there are no statistics
	 * 
	 * @return <code>true</code> if there are no statistics
	 * @see java.util.Collection#isEmpty()
	 */
	public boolean isDeckStatEmpty()
	{
		return this.testStats.isEmpty();
	}

	/**
	 * Returns the number of cards in this deck
	 * 
	 * @return the number of cards in this deck
	 */
	public int numCards()
	{
		return this.cards.size();
	}

	/**
	 * Returns the number of test statistics in this deck
	 * 
	 * @return the number of test statistics in this deck
	 * @see java.util.Collection#size()
	 */
	public int numDeckStatistics()
	{
		return this.testStats.size();
	}

	/**
	 * Removes the specified {@link Card} from the <code>Deck</code>, if it is present
	 * 
	 * @param c
	 *        the {@link Card} to be removed from the <code>Deck</code>, if it is present
	 * @see java.util.Collection#remove(Object)
	 */
	public boolean removeCard(Card c)
	{
		this.hasChanged = true;
		return this.cards.remove(c);
	}

	/**
	 * Updates the location of this <code>Deck</code> on disk
	 * 
	 * @param d
	 *        a <code>String</code> representing the new location of this <code>Deck</code> on disk
	 */
	public void setDiskLocation(String d)
	{
		this.hasChanged = true;
		this.diskLocation = d;
	}

	/**
	 * Sets the hasChanged flag to the specified <code>boolean</code>
	 * 
	 * @param b
	 *        the <code>boolean</code> to update the flag to
	 */
	public void setHasChanged(boolean b)
	{
		this.hasChanged = b;
	}

	/**
	 * Updates the name of this <code>Deck</code> to the specified <code>String</code>
	 * 
	 * @param n
	 *        a <code>String</code> representing the new name of this <code>Deck</code>
	 */
	public void setName(String n)
	{
		if(n == null || n.equals(""))
		{
			this.name = "Default Deck";
			this.hasChanged = true;
		}
		else
			this.name = n;
	}

	/**
	 * Randomizes the ordering of the cards in the card list, "shuffling" them
	 */
	public void shuffle()
	{
		if(this.cards.size() <= 1)
			return;

		ArrayList<Card> reordered = new ArrayList<Card>();
		Random rand = new Random();

		// Remove a uniformly distributed card from the current card list and add it to a temporary list
		while(this.cards.size() > 0)
			reordered.add(this.cards.remove(rand.nextInt(this.cards.size())));

		this.cards = reordered;

		this.hasChanged = true;
	}

	/**
	 * Returns an iterator over this {@code Deck}'s statistics
	 * 
	 * @return an iterator over the collection of {@link TestStat}s of this {@code Deck}
	 * @see java.util.Collection#iterator()
	 */
	public Iterator<TestStat> testStatisticsIterator()
	{
		return this.testStats.iterator();
	}

	/**
	 * A <code>String</code> representation of the entire deck (including cards).
	 */
	public String toString()
	{
		String r = "";
		r += this.name + "\n";
		r += this.diskLocation + "\n";
		r += this.hasChanged + "\n";
		for(Card c : this.cards)
			r += c.toString();
		for(TestStat t : this.testStats)
			r += t.toString();
		return r;
	}

	/**
	 * Export the contents of this <code>Deck</code> to disk
	 * 
	 * @param filepath
	 *        A <code>String</code> representing the absolute path file name of the file
	 */
	public void writeToDisk(String filepath) throws IOException
	{
		if(!filepath.endsWith(".elvl"))
			filepath += ".elvl";
		ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(filepath));
		this.diskLocation = new String(filepath);
		this.hasChanged = false;
		out.writeObject(this);
		out.close();
	}
}
