/**
 * PreferenceWindow.java
 * Group 7, Object Oriented Software Engineering
 * October 19, 2009
 */
package edu.jhu.cs.oose.elevel.preferences;

import java.util.HashMap;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.core.Qt.AlignmentFlag;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QDialogButtonBox;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QKeyEvent;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QListWidget;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QRadioButton;
import com.trolltech.qt.gui.QStackedWidget;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;
import edu.jhu.cs.oose.elevel.utilities.Algorithm;
import edu.jhu.cs.oose.elevel.utilities.PreferenceButtonGroup;
import edu.jhu.cs.oose.elevel.utilities.PreferenceCheckBox;

/**
 * Creates a Window in which the current program preference can be
 * changed and viewed
 * 
 * @author Jesse Yates, H. Parker Shelton
 */
public class PreferenceWindow extends QDialog
{
	/** QWidgets for the different preference panes */
	QWidget program;
	QWidget testing;

	/** The QLayout for the window */
	QStackedWidget stackedWidget;

	/** The QListWidget for the window */
	QListWidget listWidget;

	/** A QLabel that stores the text of the default export location */
	QLabel exportLabel;

	/**
	 * Initializes a <code>PreferenceWindow</code> with the given parent
	 * 
	 * @param parent
	 *        A <code>QWidget</code> representing the parent window
	 */
	public PreferenceWindow(QWidget parent)
	{
		super(parent);

		listWidget = new QListWidget();
		listWidget.addItem(tr("Program Settings"));
		listWidget.addItem(tr("Test Settings"));
		listWidget.setFixedWidth(120);

		QPushButton closeButton = new QPushButton(tr("Close"));
		closeButton.setDefault(true);
		QPushButton revertButton = new QPushButton(tr("Restore to Defaults"));

		QHBoxLayout buttonsLayout = new QHBoxLayout();
		buttonsLayout.addWidget(revertButton);
		buttonsLayout.addStretch(1);
		buttonsLayout.addWidget(closeButton);

		revertButton.pressed.connect(this, "restoreDefaults()");
		closeButton.pressed.connect(this, "close()");

		this.stackedWidget = new QStackedWidget();
		listWidget.currentRowChanged.connect(this.stackedWidget, "setCurrentIndex(int)");
		listWidget.setCurrentRow(0);

		this.program = createProgramPreferences();
		this.testing = createTestingPreferences();
		this.stackedWidget.addWidget(this.program);
		this.stackedWidget.addWidget(this.testing);

		QHBoxLayout horizontalLayout = new QHBoxLayout();
		horizontalLayout.addWidget(listWidget);
		horizontalLayout.addWidget(stackedWidget);

		QVBoxLayout windowLayout = new QVBoxLayout();
		windowLayout.addLayout(horizontalLayout);
		windowLayout.addLayout(buttonsLayout);
		this.setLayout(windowLayout);

		setWindowTitle(tr("Preferences"));
	}

	/**
	 * Displays a dialog to change the default export directory
	 */
	public void changeExportLocation()
	{
		Preferences.getInstance().changeExportLocation(this, "Please select a new directory for E-Level decks:");
		exportLabel.setText(truncatePath(Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION")));
	}

	/**
	 * Truncates the given string, returning the root directory and the last two directories
	 * 
	 * @param s
	 *        a <code>String</code> representing the path of a directory
	 * @return
	 *         a <code>String</code> with the first and last two directories and ellipses in the middle
	 */
	private String truncatePath(String path)
	{
		String s = path;
		if(s.length() > 15)
		{
			if(s.endsWith("/"))
				s = s.substring(0, s.length() - 1);
			String end = s.substring(s.substring(0, s.lastIndexOf("/")).lastIndexOf("/"));
			String start = s.substring(0, s.substring(1).indexOf("/") + 2);
			s = start + "..." + end;
		}
		return s;
	}

	/**
	 * Writes out the preferences when the window is closed
	 */
	public void closeEvent(QCloseEvent event)
	{
		Preferences.getInstance().writeSettings();
	}

	public void keyPressEvent(QKeyEvent event)
	{
		if(event.key() == Qt.Key.Key_Escape.value())
			this.close();
	}

	/**
	 * Initializes a widget containing the preferences for the program overall
	 */
	private QWidget createProgramPreferences()
	{
		QGridLayout layout = new QGridLayout();
		layout.addWidget(new QLabel("<b>Program Settings</b>"), 0, 0, 1, 2, new Qt.Alignment(Qt.AlignmentFlag.AlignCenter, Qt.AlignmentFlag.AlignTop));

		// Add checkbox for the splash screen
		PreferenceCheckBox splash = new PreferenceCheckBox(tr("Enable splash screen"), "SHOW_SPLASH");
		splash.setChecked(Preferences.getInstance().getValue("SHOW_SPLASH").equals("true"));
		splash.stateChange.connect(this, "updatePreference(String, boolean)");
		layout.addWidget(splash, 1, 0);

		// Add checkbox for open previously open decks
		PreferenceCheckBox openLast = new PreferenceCheckBox(tr("Open previously open decks at startup"), "OPEN_PREVIOUS");
		openLast.setChecked(Preferences.getInstance().getValue("OPEN_PREVIOUS").equals("true"));
		openLast.stateChange.connect(this, "updatePreference(String, boolean)");
		layout.addWidget(openLast, 2, 0);

		// Add text for default export location
		exportLabel = new QLabel(truncatePath(Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION")));
		layout.addWidget(exportLabel, 3, 0, Qt.AlignmentFlag.AlignLeft);

		// Add button for default export location
		QPushButton b = new QPushButton(tr("Change Export Location"), this);
		b.clicked.connect(this, "changeExportLocation()");
		layout.addWidget(b, 3, 1, Qt.AlignmentFlag.AlignCenter);

		QWidget programPage = new QWidget();
		programPage.setLayout(layout);
		return programPage;
	}

	/**
	 * Initializes a widget containing the preferences for testing
	 */
	private QWidget createTestingPreferences()
	{
		QGridLayout layout = new QGridLayout();
		layout.addWidget(new QLabel("<b>Test Settings</b>"), 0, 0, 1, 2, new Qt.Alignment(Qt.AlignmentFlag.AlignCenter, Qt.AlignmentFlag.AlignTop));

		// Add checkbox for show timer
		PreferenceCheckBox showTimer = new PreferenceCheckBox(tr("Show the timer"), "SHOW_TIMER");
		showTimer.setChecked(Preferences.getInstance().getValue("SHOW_TIMER").equals("true"));
		showTimer.stateChange.connect(this, "updatePreference(String, boolean)");
		layout.addWidget(showTimer, 1, 0, Qt.AlignmentFlag.AlignTop);

		// Add checkbox for show score
		PreferenceCheckBox showScore = new PreferenceCheckBox(tr("Show the current percent correct"), "SHOW_SCORE");
		showScore.setChecked(Preferences.getInstance().getValue("SHOW_SCORE").equals("true"));
		showScore.stateChange.connect(this, "updatePreference(String, boolean)");
		layout.addWidget(showScore, 2, 0, Qt.AlignmentFlag.AlignTop);

		// Add checkbox for allow hints
		PreferenceCheckBox allowHints = new PreferenceCheckBox(tr("Allow hints"), "ALLOW_HINTS");
		allowHints.setChecked(Preferences.getInstance().getValue("ALLOW_HINTS").equals("true"));
		allowHints.stateChange.connect(this, "updatePreference(String, boolean)");
		layout.addWidget(allowHints, 3, 0, Qt.AlignmentFlag.AlignTop);

		// Add checkbox for always show preferences
		PreferenceCheckBox alwaysShow = new PreferenceCheckBox(tr("Always show preferences on testing"), "SHOW_TEST_PREFERENCES");
		alwaysShow.setChecked(Preferences.getInstance().getValue("SHOW_TEST_PREFERENCES").equals("true"));
		alwaysShow.stateChange.connect(this, "updatePreference(String, boolean)");
		layout.addWidget(alwaysShow, 4, 0, Qt.AlignmentFlag.AlignTop);

		// Add buttons to a button group associated with the algorithm preference
		QGroupBox box = new QGroupBox("Default Testing Algorithm");
		PreferenceButtonGroup prefGroup = new PreferenceButtonGroup("DEFAULT_ALGORITHM");
		QRadioButton sequential = new QRadioButton(tr("Sequential"));
		QRadioButton random = new QRadioButton(tr("Random"));
		QRadioButton probability = new QRadioButton(tr("Probabilistic"));
		QRadioButton other = new QRadioButton(tr("Other"));
		prefGroup.addButton(sequential);
		prefGroup.addButton(random);
		prefGroup.addButton(probability);
		prefGroup.addButton(other);
		// HPS - Terrible hack because button(#) returns only null
		prefGroup.buttons().get(Algorithm.valueOf(Preferences.getInstance().getValue("DEFAULT_ALGORITHM")).ordinal()).setChecked(true);

		// Add buttons for default algorithms
		PreferenceCheckBox repeat = new PreferenceCheckBox(tr("Repeat cards"), "REPEAT_CARDS");
		repeat.setChecked(Preferences.getInstance().getValue("REPEAT_CARDS").equals("true"));
		repeat.stateChange.connect(this, "updatePreference(String, boolean)");

		QVBoxLayout algorithmLayout = new QVBoxLayout();
		algorithmLayout.addWidget(sequential);
		algorithmLayout.addWidget(random);
		algorithmLayout.addWidget(probability);
		algorithmLayout.addWidget(other);
		algorithmLayout.addWidget(repeat);
		box.setLayout(algorithmLayout);
		prefGroup.stateChange.connect(this, "updatePreference(String, String)");
		layout.addWidget(box, 1, 1, 4, 1);

		QWidget testingPage = new QWidget();
		testingPage.setLayout(layout);
		return testingPage;
	}

	/**
	 * Restores the preferences to a set of default values
	 */
	public void restoreDefaults()
	{
		if(this.listWidget.currentRow() == 0)
		{
			Preferences.getInstance().setValue("SHOW_SPLASH", "true");
			Preferences.getInstance().setValue("OPEN_PREVIOUS", "false");
			Preferences.getInstance().setDefaultLocation();
			stackedWidget.removeWidget(program);
			program = createProgramPreferences();
			stackedWidget.insertWidget(0, program);
			stackedWidget.setCurrentIndex(0);
		}
		else if(this.listWidget.currentRow() == 1)
		{
			Preferences.getInstance().setValue("SHOW_TIMER", "true");
			Preferences.getInstance().setValue("SHOW_SCORE", "true");
			Preferences.getInstance().setValue("ALLOW_HINTS", "true");
			Preferences.getInstance().setValue("SHOW_TEST_PREFERENCES", "true");
			Preferences.getInstance().setValue("DEFAULT_ALGORITHM", "SEQUENTIAL");
			Preferences.getInstance().setValue("REPEAT_CARDS", "false");
			stackedWidget.removeWidget(testing);
			testing = createTestingPreferences();
			stackedWidget.insertWidget(1, testing);
			stackedWidget.setCurrentIndex(1);
		}
	}

	/**
	 * Restores the preferences to the previous values
	 */
	public void restorePrevious()
	{
		this.close();
	}

	/**
	 * Set the preference represented by the specified key to the specified value, to be written on close.
	 * Used by the {@link PreferenceCheckBox}
	 * 
	 * @param s
	 *        A <code>String</code> representing the name of a preference
	 * @param b
	 *        A <code>boolean</code> representing the value of the preference
	 */
	public void updatePreference(String s, boolean b)
	{
		Preferences.getInstance().setValue(s, String.valueOf(b));
	}

	/**
	 * Set the preference represented by the specified key to the specified value, to be written on close.
	 * Used by the {@link PreferenceButtonGroup}
	 * 
	 * @param s
	 *        a <code>String</code> representing the name of a preference
	 * @param s2
	 *        a <code>String</code> representing the value of the preference
	 */
	public void updatePreference(String s, String s2)
	{
		Preferences.getInstance().setValue(s, s2);
	}

	/**
	 * Displays a dialog box containing test preferences at the start of the test if the global preference is set
	 * appropriately
	 */
	public static HashMap<String, String> showTestPreferences(QWidget parent)
	{
		HashMap<String, String> preferences = new HashMap<String, String>();
		preferences.putAll(Preferences.getInstance().getTestingPreferences());

		QGridLayout layout = new QGridLayout();
		layout.addWidget(new QLabel("<b>Test Settings</b>"), 0, 0, 1, 2, new Qt.Alignment(AlignmentFlag.AlignCenter, AlignmentFlag.AlignTop));

		// Add checkbox for show timer
		PreferenceCheckBox showTimer = new PreferenceCheckBox(parent.tr("Show the timer"), "SHOW_TIMER");
		showTimer.setChecked(preferences.get("SHOW_TIMER").equals("true"));
		// showTimer.stateChange.connect(test2, "updatePreference(String, boolean)");
		layout.addWidget(showTimer, 1, 0, AlignmentFlag.AlignTop);

		// Add checkbox for show score
		PreferenceCheckBox showScore = new PreferenceCheckBox(parent.tr("Show the current percent correct"), "SHOW_SCORE");
		showScore.setChecked(preferences.get("SHOW_SCORE").equals("true"));
		// showScore.stateChange.connect(test2, "updatePreference(String, boolean)");
		layout.addWidget(showScore, 2, 0, AlignmentFlag.AlignTop);

		// Add checkbox for allow hints
		PreferenceCheckBox allowHints = new PreferenceCheckBox(parent.tr("Allow hints"), "ALLOW_HINTS");
		allowHints.setChecked(preferences.get("ALLOW_HINTS").equals("true"));
		// allowHints.stateChange.connect(test2, "updatePreference(String, boolean)");
		layout.addWidget(allowHints, 3, 0, AlignmentFlag.AlignTop);

		// Add buttons for front to back and back to front
		QGroupBox group = new QGroupBox(parent.tr("Testing Direction"));
		QRadioButton backwards = new QRadioButton(parent.tr("Back to Front"));
		QRadioButton forwards = new QRadioButton(parent.tr("Front to Back"));
		forwards.setChecked(true);
		QVBoxLayout groupLayout = new QVBoxLayout();
		groupLayout.addWidget(forwards);
		groupLayout.addWidget(backwards);
		group.setLayout(groupLayout);
		layout.addWidget(group, 1, 1, 4, 1, AlignmentFlag.AlignCenter);

		// Add the appropriate buttons to a button group tied to the algorithm preference
		PreferenceButtonGroup prefGroup = new PreferenceButtonGroup("DEFAULT_ALGORITHM");
		QRadioButton sequential = new QRadioButton(parent.tr("Sequential"));
		QRadioButton random = new QRadioButton(parent.tr("Random"));
		 QRadioButton probability = new QRadioButton(parent.tr("Adaptive"));
		// QRadioButton other = new QRadioButton(parent.tr("Other"));
		prefGroup.addButton(sequential);
		prefGroup.addButton(random);
		 prefGroup.addButton(probability);
		// prefGroup.addButton(other);
		// HPS - Terrible hack because button(#) returns only null
		prefGroup.buttons().get(Algorithm.valueOf(Preferences.getInstance().getValue("DEFAULT_ALGORITHM")).ordinal()).setChecked(true);

		// Add buttons for algorithms
		PreferenceCheckBox repeat = new PreferenceCheckBox(parent.tr("Repeat cards"), "REPEAT_CARDS");
		repeat.setChecked(preferences.get("REPEAT_CARDS").equals("true"));
		// repeat.stateChange.connect(test2, "updatePreference(String, boolean)");

		QGroupBox box = new QGroupBox(parent.tr("Testing Algorithm"));
		QGridLayout algorithmLayout = new QGridLayout();
		algorithmLayout.addWidget(sequential, 1, 0);
		algorithmLayout.addWidget(random, 1, 1);
		algorithmLayout.addWidget(probability, 2, 0);
		// algorithmLayout.addWidget(other, 2, 1);
		algorithmLayout.addWidget(repeat, 3, 0);
		box.setLayout(algorithmLayout);
		layout.addWidget(box, 5, 0, 1, 2, AlignmentFlag.AlignCenter);

		// Add buttons for accept and cancel
		QDialogButtonBox buttons = new QDialogButtonBox();
		QPushButton cancel = new QPushButton("Cancel");
		QPushButton test = new QPushButton("Test");
		buttons.addButton(cancel, QDialogButtonBox.ButtonRole.RejectRole);
		buttons.addButton(test, QDialogButtonBox.ButtonRole.AcceptRole);
		layout.addWidget(buttons, 7, 0, 1, 2, AlignmentFlag.AlignCenter);

		QDialog pane = new QDialog();
		pane.setLayout(layout);

		test.clicked.connect(pane, "accept()");
		cancel.clicked.connect(pane, "reject()");
		if(pane.exec() == 1)
		{
			preferences.put("DEFAULT_ALGORITHM", Algorithm.valueOf(prefGroup.checkedButton().text().toUpperCase()).toString());
			preferences.put("REPEAT_CARDS", String.valueOf(repeat.isChecked()));
			preferences.put("SHOW_TIMER", String.valueOf(showTimer.isChecked()));
			preferences.put("SHOW_SCORE", String.valueOf(showScore.isChecked()));
			preferences.put("ALLOW_HINTS", String.valueOf(allowHints.isChecked()));
			preferences.put("TEST_FRONT_TO_BACK", String.valueOf(forwards.isChecked()));

			return preferences;
		}
		else
			return null;
	}
}
