/**
 * Test.java
 * Group 7, Object Oriented Software Engineering
 * November 15, 2009
 */
package edu.jhu.cs.oose.elevel.testTaking;

import java.util.HashMap;
import java.util.Map;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.dataModel.CardSection;
import edu.jhu.cs.oose.elevel.dataModel.Deck;
import edu.jhu.cs.oose.elevel.dataModel.TestStat;
import edu.jhu.cs.oose.elevel.utilities.ViewState;

public class Test
{
	/** The <code>Deck</code> currently being tested */
	protected Deck deck;

	/** The statistics for the current test */
	private TestStat testStat;

	/** The number of cards in the deck */
	private int numCards;

	/** The number of cards tested */
	private int cardsTested;

	/** The start time of the current test */
	protected long startTime;

	/** The set of preferences for the given test */
	private Map<String, String> preferences;

	/** A reference to ViewState */
	private ViewState state;

	private TestAlgorithm algorithm;

	/**
	 * Initializes a new <code>TestWindow</code> with the specified <code>Deck</code>
	 * 
	 * @param currentDeck
	 *        a <code>Deck</code> representing the currently selected <code>Deck</code> on which to test
	 */
	public Test(Deck currentDeck, HashMap<String, String> preferences)
	{
		this.deck = currentDeck;
		this.preferences = preferences;
		this.numCards = this.deck.numCards();
		this.state = ViewState.INSTANCE;
		initializeTest();
	}

	/**
	 * Gets the <code>String</code> value of the preference denoted by a <code>String</code>
	 * 
	 * @param s
	 *        The preference of interest.
	 * @return The value of the preference of interest.
	 */
	public String getPreference(String s)
	{
		return this.preferences.get(s);
	}

	/**
	 * Sets the currently selected <code>Deck</code> to the specified {@link Deck}
	 * 
	 * @param d
	 *        a <code>Deck</code> to assign to the currently selected deck
	 */
	public void setDeck(Deck d)
	{
		this.deck = d;
	}

	/**
	 * Check the given answer with the answer stored in the <code>Deck</code>
	 * 
	 * @param s
	 *        the <code>String</code> to check
	 * @return whether the answer is correct or not
	 */
	public boolean checkAnswer(String s)
	{
		CardSection a = state.getCurrentCard().getAnswer();
		if(a.hasText())
			checkAnswer(a.getText().equalsIgnoreCase(s));
		else
			checkAnswer(false);
		cardsTested++;

		return a.getText().equalsIgnoreCase(s);
	}

	/**
	 * Determine if the answer given to the previous {@link Card} was correct.
	 * Used for <code>Card</code>s with no computer-checkable answer.
	 * 
	 * @param answer
	 *        <code>true</code> if the previous answer was correct, <code>false</code> otherwise.
	 */
	private void checkAnswer(boolean answer)
	{
		testStat.incrementPercent(answer);
		// XXX HPS - Add card's unique ID in the deck, not the question
		if(!answer)
			testStat.addIncorrectlyAnsweredCard(state.getCurrentCard().getID());
		cardsTested++;
	}

	/**
	 * Increment the number of correct answers
	 */
	public void rightAnswer()
	{
		checkAnswer(true);
	}

	/**
	 * Increment the number of incorrect answers and adds the current card to the list of incorrectly answered cards.
	 * Note that having answered a card incorrectly once is a necessary and sufficient condition for adding to this
	 * list.
	 */
	public void wrongAnswer()
	{
		checkAnswer(false);
	}

	/**
	 * Completes the test and permanently associates the current statistics with the current <code>Deck</code>
	 */
	public TestStat finishTest()
	{
		state.nextCard();

		// XXX: HPS - probably not safe to make the cast, but that's a reeaally long time.
		this.testStat.setDuration(testTime());
		deck.addTestStat(this.testStat);
		return this.testStat;
		// Other closing things
	}

	/*
	 * Returns the % correct so far
	 */
	public String getPercentSatus()
	{
		return "" + this.testStat.getPercentCorrect() * 100 + "%";
	}

	/**
	 * Get the current duration of the test
	 * 
	 * @return
	 */
	public int testTime()
	{
		return (int) Math.round(((System.currentTimeMillis() - startTime) / 1000));
	}

	/**
	 * Initializes default parameters for a new test
	 */
	protected void initializeTest()
	{
		this.cardsTested = 0;
		this.testStat = new TestStat(System.currentTimeMillis());
		this.startTime = System.currentTimeMillis();

		// XXX: must be a better way to do this :(
		String algo = preferences.get("DEFAULT_ALGORITHM");
		boolean repeat = preferences.get("REPEAT_CARDS").equalsIgnoreCase("TRUE");

		if(algo.equals("SEQUENTIAL"))
		{
			algorithm = new SequentialTestAlgorithm(numCards, repeat);
		}
		if(algo.equals("RANDOM"))
		{
			algorithm = new RandomTestAlgorithm(numCards, repeat, null);
		}
		if(algo.equals("ADAPTIVE"))
		{
			algorithm = new AdaptiveTestAlgorithm(numCards, repeat, null);
		}

		nextCard();
	}

	/**
	 * Selects the next <code>Card</code>
	 * 
	 * @return <code>true</code> if there is a next <code>Card</code>, <code>false</code> if the test is over
	 */
	public boolean nextCard()
	{
		int currentCardIndex = algorithm.nextCard();

		if(currentCardIndex != -1)
		{
			state.setCurrentCardAndDeck(deck.getCard(currentCardIndex), deck);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Get the current <code>Card</code> from the {@link Deck}
	 * 
	 * @return the current <code>Card</code>
	 */
	public Card currentCard()
	{
		return state.getCurrentCard();
	}
}
